@@ -33,22 +33,12 @@ static void create_window (GApplication *app, const char *contents);
 static void
 show_action_dialog (GSimpleAction *action)
 {
-  const char *name;
-  GtkWidget *dialog;
+  GtkAlertDialog *dialog;
 
-  name = g_action_get_name (G_ACTION (action));
-
-  dialog = gtk_message_dialog_new (NULL,
-                                   GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_INFO,
-                                   GTK_BUTTONS_CLOSE,
-                                   "You activated action: \"%s\"",
-                                    name);
-
-  g_signal_connect (dialog, "response",
-                    G_CALLBACK (gtk_window_destroy), NULL);
-
-  gtk_widget_show (dialog);
+  dialog = gtk_alert_dialog_new ("You activated action: \"%s\"",
+                                 g_action_get_name (G_ACTION (action)));
+  gtk_alert_dialog_show (dialog, NULL);
+  g_object_unref (dialog);
 }
 
 static void
@@ -67,7 +57,7 @@ show_action_infobar (GSimpleAction *action,
   text = g_strdup_printf ("You activated radio action: \"%s\".\n"
                           "Current value: %s", name, value);
   gtk_label_set_text (GTK_LABEL (window->message), text);
-  gtk_widget_show (window->infobar);
+  gtk_widget_set_visible (window->infobar, TRUE);
   g_free (text);
 }
 
@@ -90,43 +80,38 @@ activate_new (GSimpleAction *action,
 }
 
 static void
-open_response_cb (GtkNativeDialog *dialog,
-                  int              response_id,
-                  gpointer         user_data)
+open_response_cb (GObject *source,
+                  GAsyncResult *result,
+                  gpointer user_data)
 {
-  GtkFileChooserNative *native = user_data;
-  GApplication *app = g_object_get_data (G_OBJECT (native), "app");
-  GtkWidget *message_dialog;
+  GtkFileDialog *dialog = GTK_FILE_DIALOG (source);
+  GApplication *app = G_APPLICATION (user_data);
   GFile *file;
-  char *contents;
   GError *error = NULL;
 
-  if (response_id == GTK_RESPONSE_ACCEPT)
+  file = gtk_file_dialog_open_finish (dialog, result, &error);
+  if (file)
     {
-      file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (native));
+      char *contents;
 
       if (g_file_load_contents (file, NULL, &contents, NULL, NULL, &error))
         {
           create_window (app, contents);
           g_free (contents);
         }
-      else
-        {
-          message_dialog = gtk_message_dialog_new (NULL,
-                                                   GTK_DIALOG_DESTROY_WITH_PARENT,
-                                                   GTK_MESSAGE_ERROR,
-                                                   GTK_BUTTONS_CLOSE,
-                                                   "Error loading file: \"%s\"",
-                                                   error->message);
-          g_signal_connect (message_dialog, "response",
-                            G_CALLBACK (gtk_window_destroy), NULL);
-          gtk_widget_show (message_dialog);
-          g_error_free (error);
-        }
     }
 
-  gtk_native_dialog_destroy (GTK_NATIVE_DIALOG (native));
-  g_object_unref (native);
+  if (error)
+    {
+      GtkAlertDialog *alert;
+
+      alert = gtk_alert_dialog_new ("Error loading file: \"%s\"", error->message);
+      gtk_alert_dialog_show (alert, NULL);
+      g_object_unref (alert);
+      g_error_free (error);
+    }
+
+  g_object_unref (app);
 }
 
 
@@ -136,21 +121,11 @@ activate_open (GSimpleAction *action,
                gpointer       user_data)
 {
   GApplication *app = user_data;
-  GtkFileChooserNative *native;
+  GtkFileDialog *dialog;
 
-  native = gtk_file_chooser_native_new ("Open File",
-                                        NULL,
-                                        GTK_FILE_CHOOSER_ACTION_OPEN,
-                                        "_Open",
-                                        "_Cancel");
-
-  g_object_set_data_full (G_OBJECT (native), "app", g_object_ref (app), g_object_unref);
-  g_signal_connect (native,
-                    "response",
-                    G_CALLBACK (open_response_cb),
-                    native);
-
-  gtk_native_dialog_show (GTK_NATIVE_DIALOG (native));
+  dialog = gtk_file_dialog_new ();
+  gtk_file_dialog_open (dialog, NULL, NULL, open_response_cb, g_object_ref (app));
+  g_object_unref (dialog);
 }
 
 static void
@@ -239,6 +214,41 @@ activate_quit (GSimpleAction *action,
     }
 }
 
+static void
+delete_messages (gpointer data)
+{
+  g_list_free_full ((GList *)data, g_free);
+}
+
+static void
+pop_message (GtkWidget *status)
+{
+  GList *messages = (GList *) g_object_steal_data (G_OBJECT (status), "messages");
+
+  if (messages)
+    {
+      char *message = messages->data;
+      messages = g_list_remove (messages, message);
+
+      g_object_set_data_full (G_OBJECT (status), "messages",
+                              messages, delete_messages);
+
+      gtk_label_set_label (GTK_LABEL (status), message);
+    }
+}
+
+static void
+push_message (GtkWidget  *status,
+              const char *message)
+{
+  GList *messages = (GList *) g_object_steal_data (G_OBJECT (status), "messages");
+
+  gtk_label_set_label (GTK_LABEL (status), message);
+  messages = g_list_prepend (messages, g_strdup (message));
+  g_object_set_data_full (G_OBJECT (status), "messages",
+                          messages, delete_messages);
+}
+
 static void
 update_statusbar (GtkTextBuffer         *buffer,
                   DemoApplicationWindow *window)
@@ -249,7 +259,7 @@ update_statusbar (GtkTextBuffer         *buffer,
   GtkTextIter iter;
 
   /* clear any previous message, underflow is allowed */
-  gtk_statusbar_pop (GTK_STATUSBAR (window->status), 0);
+  pop_message (window->status);
 
   count = gtk_text_buffer_get_char_count (buffer);
 
@@ -263,7 +273,7 @@ update_statusbar (GtkTextBuffer         *buffer,
   msg = g_strdup_printf ("Cursor at row %d column %d - %d chars in document",
                          row, col, count);
 
-  gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, msg);
+  push_message (window->status, msg);
 
   g_free (msg);
 }
@@ -320,7 +330,7 @@ static GActionEntry win_entries[] = {
 static void
 clicked_cb (GtkWidget *widget, DemoApplicationWindow *window)
 {
-  gtk_widget_hide (window->infobar);
+  gtk_widget_set_visible (window->infobar, FALSE);
 }
 
 static void
