@@ -21,8 +21,8 @@
 
 #include "gdkmemorytextureprivate.h"
 
+#include "gdkcolorstateprivate.h"
 #include "gdkmemoryformatprivate.h"
-#include "gsk/gl/fp16private.h"
 
 /**
  * GdkMemoryTexture:
@@ -58,6 +58,7 @@ gdk_memory_texture_dispose (GObject *object)
 static void
 gdk_memory_texture_download (GdkTexture      *texture,
                              GdkMemoryFormat  format,
+                             GdkColorState   *color_state,
                              guchar          *data,
                              gsize            stride)
 {
@@ -65,9 +66,11 @@ gdk_memory_texture_download (GdkTexture      *texture,
 
   gdk_memory_convert (data, stride,
                       format,
+                      color_state,
                       (guchar *) g_bytes_get_data (self->bytes, NULL),
                       self->stride,
                       texture->format,
+                      texture->color_state,
                       gdk_texture_get_width (texture),
                       gdk_texture_get_height (texture));
 }
@@ -108,7 +111,7 @@ gdk_memory_sanitize (GBytes          *bytes,
       stride % align == 0)
     {
       *out_stride = stride;
-      return g_bytes_ref (bytes);
+      return bytes;
     }
 
   bpp = gdk_memory_format_bytes_per_pixel (format);
@@ -119,10 +122,52 @@ gdk_memory_sanitize (GBytes          *bytes,
   for (y = 0; y < height; y++)
     memcpy (copy + y * copy_stride, data + y * stride, bpp * width);
 
+  g_bytes_unref (bytes);
+
   *out_stride = copy_stride;
   return g_bytes_new_take (copy, copy_stride * height);
 }
 
+GdkTexture *
+gdk_memory_texture_new_from_builder (GdkMemoryTextureBuilder *builder)
+{
+  GdkMemoryTexture *self;
+  GdkTexture *texture, *update_texture;
+
+  self = g_object_new (GDK_TYPE_MEMORY_TEXTURE,
+                       "width", gdk_memory_texture_builder_get_width (builder),
+                       "height", gdk_memory_texture_builder_get_height (builder),
+                       "color-state", gdk_memory_texture_builder_get_color_state (builder),
+                       NULL);
+  texture = GDK_TEXTURE (self);
+
+  texture->format = gdk_memory_texture_builder_get_format (builder);
+  self->bytes = gdk_memory_sanitize (g_bytes_ref (gdk_memory_texture_builder_get_bytes (builder)),
+                                     texture->width,
+                                     texture->height,
+                                     texture->format,
+                                     gdk_memory_texture_builder_get_stride (builder),
+                                     &self->stride);
+
+  update_texture = gdk_memory_texture_builder_get_update_texture (builder);
+  if (update_texture)
+    {
+      cairo_region_t *update_region = gdk_memory_texture_builder_get_update_region (builder);
+      if (update_region)
+        {
+          update_region = cairo_region_copy (update_region);
+          cairo_region_intersect_rectangle (update_region,
+                                            &(cairo_rectangle_int_t) {
+                                              0, 0,
+                                              update_texture->width, update_texture->height
+                                            });
+          gdk_texture_set_diff (GDK_TEXTURE (self), update_texture, update_region);
+        }
+    }
+
+  return GDK_TEXTURE (self);
+}
+
 /**
  * gdk_memory_texture_new:
  * @width: the width of the texture
@@ -152,13 +197,14 @@ gdk_memory_texture_new (int              width,
   g_return_val_if_fail (bytes != NULL, NULL);
   g_return_val_if_fail (stride >= width * gdk_memory_format_bytes_per_pixel (format), NULL);
   /* needs to be this complex to support subtexture of the bottom right part */
-  g_return_val_if_fail (g_bytes_get_size (bytes) >= stride * (height - 1) + width * gdk_memory_format_bytes_per_pixel (format), NULL);
+  g_return_val_if_fail (g_bytes_get_size (bytes) >= gdk_memory_format_min_buffer_size (format, stride, width, height), NULL);
 
-  bytes = gdk_memory_sanitize (bytes, width, height, format, stride, &stride);
+  bytes = gdk_memory_sanitize (g_bytes_ref (bytes), width, height, format, stride, &stride);
 
   self = g_object_new (GDK_TYPE_MEMORY_TEXTURE,
                        "width", width,
                        "height", height,
+                       "color-state", GDK_COLOR_STATE_SRGB,
                        NULL);
 
   GDK_TEXTURE (self)->format = format;
@@ -176,7 +222,7 @@ gdk_memory_texture_new_subtexture (GdkMemoryTexture  *source,
                                    int                height)
 {
   GdkTexture *texture, *result;
-  gsize bpp, offset, size;
+  gsize offset, size;
   GBytes *bytes;
 
   g_return_val_if_fail (GDK_IS_MEMORY_TEXTURE (source), NULL);
@@ -186,9 +232,8 @@ gdk_memory_texture_new_subtexture (GdkMemoryTexture  *source,
   g_return_val_if_fail (height > 0 && y + height <= GDK_TEXTURE (source)->height, NULL);
 
   texture = GDK_TEXTURE (source);
-  bpp = gdk_memory_format_bytes_per_pixel (texture->format);
-  offset = y * source->stride + x * bpp;
-  size = source->stride * (height - 1) + width * bpp;
+  offset = y * source->stride + x * gdk_memory_format_bytes_per_pixel (texture->format);
+  size = gdk_memory_format_min_buffer_size (texture->format, source->stride, width, height);
   bytes = g_bytes_new_from_bytes (source->bytes, offset, size);
 
   result = gdk_memory_texture_new (width,
@@ -202,8 +247,7 @@ gdk_memory_texture_new_subtexture (GdkMemoryTexture  *source,
 }
 
 GdkMemoryTexture *
-gdk_memory_texture_from_texture (GdkTexture      *texture,
-                                 GdkMemoryFormat  format)
+gdk_memory_texture_from_texture (GdkTexture *texture)
 {
   GdkTexture *result;
   GBytes *bytes;
@@ -213,21 +257,16 @@ gdk_memory_texture_from_texture (GdkTexture      *texture,
   g_return_val_if_fail (GDK_IS_TEXTURE (texture), NULL);
 
   if (GDK_IS_MEMORY_TEXTURE (texture))
-    {
-      GdkMemoryTexture *memtex = GDK_MEMORY_TEXTURE (texture);
+    return g_object_ref (GDK_MEMORY_TEXTURE (texture));
 
-      if (gdk_texture_get_format (texture) == format)
-        return g_object_ref (memtex);
-    }
-
-  stride = texture->width * gdk_memory_format_bytes_per_pixel (format);
+  stride = texture->width * gdk_memory_format_bytes_per_pixel (texture->format);
   data = g_malloc_n (stride, texture->height);
 
-  gdk_texture_do_download (texture, format, data, stride);
+  gdk_texture_do_download (texture, texture->format, texture->color_state, data, stride);
   bytes = g_bytes_new_take (data, stride * texture->height);
   result = gdk_memory_texture_new (texture->width,
                                    texture->height,
-                                   format,
+                                   texture->format,
                                    bytes,
                                    stride);
   g_bytes_unref (bytes);
