@@ -54,18 +54,24 @@ shm_pool_create(struct wl_shm *shm, int size)
 	struct shm_pool *pool;
 
 	pool = malloc(sizeof *pool);
-	if (!pool)
+	if (!pool) {
+                g_warning ("malloc() failed");
 		return NULL;
+        }
 
 	pool->fd = os_create_anonymous_file (size);
-	if (pool->fd < 0)
+	if (pool->fd < 0) {
+                g_warning ("os_create_anonymous_file() failed");
 		goto err_free;
+        }
 
 	pool->data = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED,
 			  pool->fd, 0);
 
-	if (pool->data == MAP_FAILED)
+	if (pool->data == MAP_FAILED) {
+                g_warning ("mmap() failed: %s", strerror (errno));
 		goto err_close;
+        }
 
 	pool->pool = wl_shm_create_pool(shm, pool->fd, size);
 	pool->size = size;
@@ -202,21 +208,16 @@ wl_cursor_destroy(struct wl_cursor *cursor)
 static struct wl_cursor *
 wl_cursor_create_from_xcursor_images(struct wl_cursor_theme *theme,
                                      const char *name,
-                                     unsigned int size,
-                                     unsigned int scale)
+                                     unsigned int size)
 {
         char *path;
         XcursorImages *images;
         struct wl_cursor *cursor;
 	struct cursor_image *image;
 	int i, nbytes;
-        unsigned int load_size;
-        int load_scale = 1;
-
-        load_size = size * scale;
 
         path = g_strconcat (theme->path, "/", name, NULL);
-        images = xcursor_load_images (path, load_size);
+        images = xcursor_load_images (path, size);
 
         if (!images)
           {
@@ -224,14 +225,6 @@ wl_cursor_create_from_xcursor_images(struct wl_cursor_theme *theme,
             return NULL;
           }
 
-        if (images->images[0]->width != load_size ||
-            images->images[0]->height != load_size)
-          {
-	        xcursor_images_destroy (images);
-                images = xcursor_load_images (path, size);
-                load_scale = scale;
-          }
-
         g_free (path);
 
 	cursor = malloc(sizeof *cursor);
@@ -249,7 +242,7 @@ wl_cursor_create_from_xcursor_images(struct wl_cursor_theme *theme,
 	}
 
 	cursor->name = strdup(name);
-        cursor->size = load_size;
+        cursor->size = images->images[0]->size;
 
 	for (i = 0; i < images->nimage; i++) {
 		image = malloc(sizeof *image);
@@ -259,10 +252,10 @@ wl_cursor_create_from_xcursor_images(struct wl_cursor_theme *theme,
 		image->theme = theme;
 		image->buffer = NULL;
 
-		image->image.width = images->images[i]->width * load_scale;
-		image->image.height = images->images[i]->height * load_scale;
-		image->image.hotspot_x = images->images[i]->xhot * load_scale;
-		image->image.hotspot_y = images->images[i]->yhot * load_scale;
+		image->image.width = images->images[i]->width;
+		image->image.height = images->images[i]->height;
+		image->image.hotspot_x = images->images[i]->xhot;
+		image->image.hotspot_y = images->images[i]->yhot;
 		image->image.delay = images->images[i]->delay;
 
 		nbytes = image->image.width * image->image.height * 4;
@@ -272,24 +265,9 @@ wl_cursor_create_from_xcursor_images(struct wl_cursor_theme *theme,
 			break;
 		}
 
-                if (load_scale == 1) {
-		    /* copy pixels to shm pool */
-                    memcpy(theme->pool->data + image->offset,
-                           images->images[i]->pixels, nbytes);
-                }
-                else {
-                    /* scale image up while copying it */
-                    for (int y = 0; y < image->image.height; y++) {
-                        char *p = theme->pool->data + image->offset + y * image->image.width * 4;
-                        char *q = ((char *)images->images[i]->pixels) + (y / load_scale) * images->images[i]->width * 4;
-                        for (int x = 0; x < image->image.width; x++) {
-                            p[4 * x] = q[4 * (x/load_scale)];
-                            p[4 * x + 1] = q[4 * (x/load_scale) + 1];
-                            p[4 * x + 2] = q[4 * (x/load_scale) + 2];
-                            p[4 * x + 3] = q[4 * (x/load_scale) + 3];
-                        }
-                    }
-                }
+		/* copy pixels to shm pool */
+                memcpy(theme->pool->data + image->offset,
+                       images->images[i]->pixels, nbytes);
 		cursor->images[i] = (struct wl_cursor_image *) image;
 	}
 	cursor->image_count = i;
@@ -310,12 +288,11 @@ wl_cursor_create_from_xcursor_images(struct wl_cursor_theme *theme,
 static void
 load_cursor(struct wl_cursor_theme *theme,
             const char             *name,
-            unsigned int            size,
-            unsigned int            scale)
+            unsigned int            size)
 {
 	struct wl_cursor *cursor;
 
-        cursor = wl_cursor_create_from_xcursor_images(theme, name, size, scale);
+        cursor = wl_cursor_create_from_xcursor_images(theme, name, size);
 
 	if (cursor) {
 		theme->cursor_count++;
@@ -349,8 +326,10 @@ wl_cursor_theme_create(const char *path, int size, struct wl_shm *shm)
 	struct wl_cursor_theme *theme;
 
 	theme = malloc(sizeof *theme);
-	if (!theme)
+	if (!theme) {
+                g_warning ("malloc() failed");
 		return NULL;
+        }
 
 	theme->path = strdup (path);
 	theme->size = size;
@@ -359,6 +338,7 @@ wl_cursor_theme_create(const char *path, int size, struct wl_shm *shm)
 
 	theme->pool = shm_pool_create(shm, size * size * 4);
 	if (!theme->pool) {
+                g_warning ("shm_pool_create() failed");
                 free (theme->path);
                 free (theme);
 		return NULL;
@@ -396,12 +376,12 @@ wl_cursor_theme_destroy(struct wl_cursor_theme *theme)
 struct wl_cursor *
 wl_cursor_theme_get_cursor(struct wl_cursor_theme *theme,
 			   const char *name,
-                           unsigned int scale)
+                           float scale)
 {
 	unsigned int i;
         unsigned int size;
 
-        size = theme->size * scale;
+        size = ceil (theme->size * scale);
 
 	for (i = 0; i < theme->cursor_count; i++) {
                 if (size == theme->cursors[i]->size &&
@@ -409,11 +389,10 @@ wl_cursor_theme_get_cursor(struct wl_cursor_theme *theme,
 		        return theme->cursors[i];
         }
 
-        load_cursor (theme, name, theme->size, scale);
+        load_cursor (theme, name, size);
 
         if (i < theme->cursor_count) {
-                if (size == theme->cursors[i]->size &&
-                    strcmp (name, theme->cursors[theme->cursor_count - 1]->name) == 0)
+                if (strcmp (name, theme->cursors[theme->cursor_count - 1]->name) == 0)
                         return theme->cursors[theme->cursor_count - 1];
         }
 
