@@ -22,6 +22,8 @@
 #include "gdkcursorprivate.h"
 #include "gdkeventsprivate.h"
 
+#define  GTK_SHELL_FIXED_WL_SURFACE_OFFSET_VERSION 6
+
 #define GDK_SLOT_TO_EVENT_SEQUENCE(s) ((GdkEventSequence *) GUINT_TO_POINTER((s) + 1))
 #define GDK_EVENT_SEQUENCE_TO_SLOT(s) (GPOINTER_TO_UINT(s) - 1)
 
@@ -252,6 +254,78 @@ gdk_wayland_device_set_emulating_touch (GdkWaylandDevice    *wayland_device,
   priv->emulating_touch = touch;
 }
 
+
+static const struct
+{
+  const char *cursor_name;
+  unsigned int shape;
+  unsigned int version;
+} shape_map[] = {
+  { "default", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_DEFAULT, 1 },
+  { "context-menu", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_CONTEXT_MENU, 1 },
+  { "help", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_HELP, 1 },
+  { "pointer", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_POINTER, 1 },
+  { "progress", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_PROGRESS },
+  { "wait", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_WAIT, 1 },
+  { "cell", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_CELL, 1 },
+  { "crosshair", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_CROSSHAIR, 1 },
+  { "text", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_TEXT, 1 },
+  { "vertical-text", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_VERTICAL_TEXT, 1 },
+  { "alias", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_ALIAS, 1 },
+  { "copy", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_COPY, 1 },
+  { "move", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_MOVE, 1 },
+  { "dnd-move", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_MOVE, 1 },
+  { "no-drop", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_NO_DROP, 1 },
+  { "not-allowed", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_NOT_ALLOWED, 1 },
+  { "grab", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_GRAB, 1 },
+  { "grabbing", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_GRABBING, 1 },
+  { "e-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_E_RESIZE, 1 },
+  { "n-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_N_RESIZE, 1 },
+  { "ne-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_NE_RESIZE, 1 },
+  { "nw-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_NW_RESIZE, 1 },
+  { "s-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_S_RESIZE, 1 },
+  { "se-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_SE_RESIZE, 1 },
+  { "sw-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_SW_RESIZE, 1 },
+  { "w-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_W_RESIZE, 1 },
+  { "ew-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_EW_RESIZE, 1 },
+  { "ns-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_NS_RESIZE, 1 },
+  { "nesw-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_NESW_RESIZE, 1 },
+  { "nwse-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_NWSE_RESIZE, 1 },
+  { "col-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_COL_RESIZE, 1 },
+  { "row-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_ROW_RESIZE, 1 },
+  { "all-scroll", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_ALL_SCROLL, 1 },
+  { "zoom-in", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_ZOOM_IN, 1 },
+  { "zoom-out", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_ZOOM_OUT, 1 },
+  { "all-scroll", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_ALL_SCROLL, 1 },
+  /* the following a v2 additions, with a fallback for v1 */
+  { "dnd-ask", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_DND_ASK, 2 },
+  { "dnd-ask", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_CONTEXT_MENU, 1 },
+  { "all-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_ALL_RESIZE, 2 },
+  { "all-resize", WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_MOVE, 1 },
+};
+
+static unsigned int
+_gdk_wayland_cursor_get_shape (GdkCursor *cursor,
+                               int        version)
+{
+  gsize i;
+  const char *cursor_name;
+
+  cursor_name = gdk_cursor_get_name (cursor);
+  if (cursor_name == NULL ||
+      g_str_equal (cursor_name, "none"))
+    return 0;
+
+  for (i = 0; i < G_N_ELEMENTS (shape_map); i++)
+    {
+      if (g_str_equal (shape_map[i].cursor_name, cursor_name) &&
+          version >= shape_map[i].version)
+        return shape_map[i].shape;
+    }
+
+  return 0;
+}
+
 gboolean
 gdk_wayland_device_update_surface_cursor (GdkDevice *device)
 {
@@ -259,33 +333,74 @@ gdk_wayland_device_update_surface_cursor (GdkDevice *device)
   GdkWaylandDevice *wayland_device = GDK_WAYLAND_DEVICE (device);
   GdkWaylandPointerData *pointer =
     gdk_wayland_device_get_pointer (wayland_device);
+  GdkWaylandDisplay *wayland_display = GDK_WAYLAND_DISPLAY (seat->display);
   struct wl_buffer *buffer;
   int x, y, w, h;
   double preferred_scale, scale;
   guint next_image_index, next_image_delay;
   gboolean retval = G_SOURCE_REMOVE;
   GdkWaylandTabletData *tablet;
+  unsigned int shape;
+  gboolean use_surface_offset;
 
   tablet = gdk_wayland_seat_find_tablet (seat, device);
 
-  preferred_scale = gdk_fractional_scale_to_double (&pointer->preferred_scale);
-
-  if (pointer->cursor)
-    {
-      buffer = _gdk_wayland_cursor_get_buffer (GDK_WAYLAND_DISPLAY (seat->display),
-                                               pointer->cursor,
-                                               preferred_scale,
-                                               pointer->cursor_image_index,
-                                               &x, &y, &w, &h,
-                                               &scale);
-    }
-  else
+  if (!pointer->cursor)
     {
       pointer->cursor_timeout_id = 0;
       return G_SOURCE_REMOVE;
     }
 
-  if (pointer->has_cursor_surface)
+  if (tablet && !tablet->current_tool)
+    {
+      pointer->cursor_timeout_id = 0;
+      return G_SOURCE_REMOVE;
+    }
+
+  if (GDK_WAYLAND_DISPLAY (seat->display)->cursor_shape)
+    {
+      shape = _gdk_wayland_cursor_get_shape (pointer->cursor,
+                                             wp_cursor_shape_manager_v1_get_version (GDK_WAYLAND_DISPLAY (seat->display)->cursor_shape));
+      if (shape != 0)
+        {
+          if (pointer->cursor_shape == shape)
+            return G_SOURCE_REMOVE;
+
+          if (tablet && tablet->current_tool->shape_device)
+            {
+              pointer->has_cursor_surface = FALSE;
+              pointer->cursor_shape = shape;
+              wp_cursor_shape_device_v1_set_shape (tablet->current_tool->shape_device, pointer->enter_serial, shape);
+              return G_SOURCE_REMOVE;
+            }
+          else if (seat->wl_pointer && pointer->shape_device)
+            {
+              pointer->has_cursor_surface = FALSE;
+              pointer->cursor_shape = shape;
+              wp_cursor_shape_device_v1_set_shape (pointer->shape_device, pointer->enter_serial, shape);
+              return G_SOURCE_REMOVE;
+            }
+        }
+    }
+
+  preferred_scale = gdk_fractional_scale_to_double (&pointer->preferred_scale);
+
+  buffer = _gdk_wayland_cursor_get_buffer (GDK_WAYLAND_DISPLAY (seat->display),
+                                           pointer->cursor,
+                                           preferred_scale,
+                                           pointer->cursor_image_index,
+                                           &x, &y, &w, &h,
+                                           &scale);
+
+  use_surface_offset =
+    pointer->has_cursor_surface &&
+    wl_surface_get_version (pointer->pointer_surface) >=
+    WL_SURFACE_OFFSET_SINCE_VERSION &&
+    (!wayland_display->gtk_shell ||
+     gtk_shell1_get_version (wayland_display->gtk_shell) >=
+     GTK_SHELL_FIXED_WL_SURFACE_OFFSET_VERSION);
+
+  if (use_surface_offset)
     {
       /* We already have the surface attached to the cursor, change the
        * offset to adapt to the new buffer.
@@ -316,16 +431,10 @@ gdk_wayland_device_update_surface_cursor (GdkDevice *device)
       wl_surface_commit (pointer->pointer_surface);
     }
 
-  if (!pointer->has_cursor_surface)
+  if (!use_surface_offset)
     {
       if (tablet)
         {
-          if (!tablet->current_tool)
-            {
-              pointer->cursor_timeout_id = 0;
-              return G_SOURCE_REMOVE;
-            }
-
           zwp_tablet_tool_v2_set_cursor (tablet->current_tool->wp_tablet_tool,
                                          pointer->enter_serial,
                                          pointer->pointer_surface,
@@ -345,6 +454,7 @@ gdk_wayland_device_update_surface_cursor (GdkDevice *device)
         }
 
       pointer->has_cursor_surface = TRUE;
+      pointer->cursor_shape = 0;
     }
 
   pointer->cursor_hotspot_x = x;
