@@ -2,6 +2,10 @@
 
 #include "gdksubsurfaceprivate.h"
 
+#include "gdkwaylandcolor-private.h"
+#include "gdkdisplay-wayland.h"
+#include "wayland-client-protocol.h"
+
 
 typedef struct _GdkWaylandSubsurface GdkWaylandSubsurface;
 typedef struct _GdkWaylandSubsurfaceClass GdkWaylandSubsurfaceClass;
@@ -19,14 +23,24 @@ struct _GdkWaylandSubsurface
   struct wl_surface *surface;
   struct wl_subsurface *subsurface;
   struct wp_viewport *viewport;
+  GdkWaylandColorSurface *color;
 
   GdkTexture *texture;
   cairo_rectangle_int_t dest;
   graphene_rect_t source;
+  enum wl_output_transform transform;
 
   struct wl_region *opaque_region;
 
   struct wl_callback *frame_callback;
+
+  struct wl_surface *bg_surface;
+  struct wl_subsurface *bg_subsurface;
+  struct wp_viewport *bg_viewport;
+  cairo_rectangle_int_t bg_rect;
+  gboolean bg_attached;
+
+  struct zwp_idle_inhibitor_v1 *idle_inhibitor;
 };
 
 struct _GdkWaylandSubsurfaceClass
@@ -41,3 +55,5 @@ void gdk_wayland_subsurface_clear_frame_callback (GdkSubsurface *subsurface);
 
 GdkSubsurface * gdk_wayland_surface_create_subsurface (GdkSurface *surface);
 
+gboolean gdk_wayland_subsurface_inhibit_idle   (GdkSubsurface *subsurface);
+void     gdk_wayland_subsurface_uninhibit_idle (GdkSubsurface *subsurface);
