@@ -32,8 +32,7 @@
 G_DEFINE_TYPE (GdkWin32CairoContext, gdk_win32_cairo_context, GDK_TYPE_CAIRO_CONTEXT)
 
 static cairo_surface_t *
-create_cairo_surface_for_surface (GdkSurface *surface,
-                                  int         scale)
+create_cairo_surface_for_surface (GdkSurface *surface)
 {
   cairo_surface_t *cairo_surface;
   HDC hdc;
@@ -46,38 +45,37 @@ create_cairo_surface_for_surface (GdkSurface *surface,
     }
 
   cairo_surface = cairo_win32_surface_create_with_format (hdc, CAIRO_FORMAT_ARGB32);
-  cairo_surface_set_device_scale (cairo_surface, scale, scale);
 
   return cairo_surface;
 }
 
 static void
-gdk_win32_cairo_context_begin_frame (GdkDrawContext *draw_context,
-                                     GdkMemoryDepth  depth,
-                                     cairo_region_t *region)
+gdk_win32_cairo_context_begin_frame (GdkDrawContext  *draw_context,
+                                     GdkMemoryDepth   depth,
+                                     cairo_region_t  *region,
+                                     GdkColorState  **out_color_state,
+                                     GdkMemoryDepth  *out_depth)
 {
   GdkWin32CairoContext *self = GDK_WIN32_CAIRO_CONTEXT (draw_context);
   GdkSurface *surface;
-  int scale;
   cairo_t *cr;
   int width, height;
-  RECT queued_window_rect;
+  RECT queued_hwnd_rect;
 
   surface = gdk_draw_context_get_surface (draw_context);
-  scale = gdk_surface_get_scale_factor (surface);
 
-  queued_window_rect = gdk_win32_surface_handle_queued_move_resize (draw_context);
+  queued_hwnd_rect = gdk_win32_surface_handle_queued_move_resize (draw_context);
 
-  width = queued_window_rect.right - queued_window_rect.left;
-  height = queued_window_rect.bottom - queued_window_rect.top;
+  width = queued_hwnd_rect.right - queued_hwnd_rect.left;
+  height = queued_hwnd_rect.bottom - queued_hwnd_rect.top;
   width = MAX (width, 1);
   height = MAX (height, 1);
 
-  self->window_surface = create_cairo_surface_for_surface (surface, scale);
+  self->hwnd_surface = create_cairo_surface_for_surface (surface);
 
   if (!self->double_buffered)
-    /* Non-double-buffered windows paint on the window surface directly */
-    self->paint_surface = cairo_surface_reference (self->window_surface);
+    /* Non-double-buffered GDK surfaces paint on the window surface directly */
+    self->paint_surface = cairo_surface_reference (self->hwnd_surface);
   else
     {
       if (width > self->db_width ||
@@ -90,13 +88,14 @@ gdk_win32_cairo_context_begin_frame (GdkDrawContext *draw_context,
 
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
           self->db_surface = gdk_surface_create_similar_surface (surface,
-                                                                 cairo_surface_get_content (self->window_surface),
+                                                                 cairo_surface_get_content (self->hwnd_surface),
                                                                  self->db_width,
                                                                  self->db_height);
 G_GNUC_END_IGNORE_DEPRECATIONS
+          cairo_surface_set_device_scale (self->db_surface, 1.0, 1.0);
         }
 
-      /* Double-buffered windows paint on a DB surface.
+      /* Double-buffered GDK surfaces paint on a DB surface.
        * Due to performance concerns we don't recreate it unless forced to.
        */
       self->paint_surface = cairo_surface_reference (self->db_surface);
@@ -115,6 +114,9 @@ G_GNUC_END_IGNORE_DEPRECATIONS
   cairo_clip (cr);
   cairo_paint (cr);
   cairo_destroy (cr);
+
+  *out_color_state = GDK_COLOR_STATE_SRGB;
+  *out_depth = gdk_color_state_get_depth (GDK_COLOR_STATE_SRGB);
 }
 
 static void
@@ -123,19 +125,19 @@ gdk_win32_cairo_context_end_frame (GdkDrawContext *draw_context,
 {
   GdkWin32CairoContext *self = GDK_WIN32_CAIRO_CONTEXT (draw_context);
 
-  /* The code to resize double-buffered windows immediately
+  /* The code to resize double-buffered GDK surfaces immediately
    * before blitting the buffer contents onto them used
    * to be here.
    */
 
-  /* For double-buffered windows we need to blit
-   * the DB buffer contents into the window itself.
+  /* For double-buffered GDK surfaces we need to blit
+   * the DB buffer contents into the GDK surface itself.
    */
   if (self->double_buffered)
     {
       cairo_t *cr;
 
-      cr = cairo_create (self->window_surface);
+      cr = cairo_create (self->hwnd_surface);
 
       cairo_set_source_surface (cr, self->paint_surface, 0, 0);
       gdk_cairo_region (cr, painted);
@@ -147,10 +149,10 @@ gdk_win32_cairo_context_end_frame (GdkDrawContext *draw_context,
       cairo_destroy (cr);
     }
 
-  cairo_surface_flush (self->window_surface);
+  cairo_surface_flush (self->hwnd_surface);
 
   g_clear_pointer (&self->paint_surface, cairo_surface_destroy);
-  g_clear_pointer (&self->window_surface, cairo_surface_destroy);
+  g_clear_pointer (&self->hwnd_surface, cairo_surface_destroy);
 }
 
 static void
