@@ -125,12 +125,12 @@ gdk_win32_gl_context_wgl_end_frame (GdkDrawContext *draw_context,
            * conventions. Coordinates are that of the client-area, but the origin is
            * at the lower-left corner; rectangles are passed by their lower-left corner
            */
-          rectangle.y = surface->height - rectangle.y - rectangle.height;
+          rectangle.y = (surface->height * scale) - rectangle.y - rectangle.height;
 
-          context_wgl->ptr_glAddSwapHintRectWIN (rectangle.x * scale,
-                                                 rectangle.y * scale,
-                                                 rectangle.width * scale,
-                                                 rectangle.height * scale);
+          context_wgl->ptr_glAddSwapHintRectWIN (rectangle.x,
+                                                 rectangle.y,
+                                                 rectangle.width,
+                                                 rectangle.height);
         }
     }
 
