@@ -3,6 +3,7 @@
 #include "gskgldeviceprivate.h"
 
 #include "gskdebugprivate.h"
+#include "gskgpushaderflagsprivate.h"
 #include "gskgpushaderopprivate.h"
 #include "gskglbufferprivate.h"
 #include "gskglimageprivate.h"
@@ -34,9 +35,9 @@ typedef struct _GLProgramKey GLProgramKey;
 struct _GLProgramKey
 {
   const GskGpuShaderOpClass *op_class;
+  GskGpuShaderFlags flags;
+  GskGpuColorStates color_states;
   guint32 variation;
-  GskGpuShaderClip clip;
-  guint n_external_textures;
 };
 
 G_DEFINE_TYPE (GskGLDevice, gsk_gl_device, GSK_TYPE_GPU_DEVICE)
@@ -47,9 +48,9 @@ gl_program_key_hash (gconstpointer data)
   const GLProgramKey *key = data;
 
   return GPOINTER_TO_UINT (key->op_class) ^
-         key->clip ^
-         (key->variation << 2) ^
-         (key->n_external_textures << 24);
+         ((key->flags << 11) | (key->flags >> 21)) ^
+         ((key->variation << 21) | ( key->variation >> 11)) ^
+         key->color_states;
 }
 
 static gboolean
@@ -60,22 +61,24 @@ gl_program_key_equal (gconstpointer a,
   const GLProgramKey *keyb = b;
 
   return keya->op_class == keyb->op_class &&
-         keya->variation == keyb->variation && 
-         keya->clip == keyb->clip && 
-         keya->n_external_textures == keyb->n_external_textures;
+         keya->flags == keyb->flags && 
+         keya->color_states == keyb->color_states && 
+         keya->variation == keyb->variation;
 }
 
 static GskGpuImage *
 gsk_gl_device_create_offscreen_image (GskGpuDevice   *device,
                                       gboolean        with_mipmap,
-                                      GdkMemoryDepth  depth,
+                                      GdkMemoryFormat format,
+                                      gboolean        is_srgb,
                                       gsize           width,
                                       gsize           height)
 {
   GskGLDevice *self = GSK_GL_DEVICE (device);
 
   return gsk_gl_image_new (self,
-                           gdk_memory_depth_get_format (depth),
+                           format,
+                           is_srgb,
                            GSK_GPU_IMAGE_RENDERABLE | GSK_GPU_IMAGE_FILTERABLE,
                            width,
                            height);
@@ -85,6 +88,7 @@ static GskGpuImage *
 gsk_gl_device_create_upload_image (GskGpuDevice    *device,
                                    gboolean         with_mipmap,
                                    GdkMemoryFormat  format,
+                                   gboolean         try_srgb,
                                    gsize            width,
                                    gsize            height)
 {
@@ -92,6 +96,7 @@ gsk_gl_device_create_upload_image (GskGpuDevice    *device,
 
   return gsk_gl_image_new (self,
                            format,
+                           try_srgb,
                            0,
                            width,
                            height);
@@ -107,6 +112,7 @@ gsk_gl_device_create_download_image (GskGpuDevice   *device,
 
   return gsk_gl_image_new (self,
                            gdk_memory_depth_get_format (depth),
+                           gdk_memory_depth_is_srgb (depth),
                            GSK_GPU_IMAGE_RENDERABLE,
                            width,
                            height);
@@ -121,6 +127,7 @@ gsk_gl_device_create_atlas_image (GskGpuDevice *device,
 
   return gsk_gl_image_new (self,
                            GDK_MEMORY_DEFAULT,
+                           FALSE,
                            GSK_GPU_IMAGE_RENDERABLE,
                            width,
                            height);
@@ -228,7 +235,7 @@ gsk_gl_device_get_for_display (GdkDisplay  *display,
 {
   GskGLDevice *self;
   GdkGLContext *context;
-  GLint max_texture_size;
+  GLint max_texture_size, globals_alignment;
 
   self = g_object_get_data (G_OBJECT (display), "-gsk-gl-device");
   if (self)
@@ -252,7 +259,13 @@ gsk_gl_device_get_for_display (GdkDisplay  *display,
   gdk_gl_context_make_current (context);
 
   glGetIntegerv (GL_MAX_TEXTURE_SIZE, &max_texture_size);
-  gsk_gpu_device_setup (GSK_GPU_DEVICE (self), display, max_texture_size);
+  glGetIntegerv (GL_UNIFORM_BUFFER_OFFSET_ALIGNMENT, &globals_alignment);
+
+  gsk_gpu_device_setup (GSK_GPU_DEVICE (self),
+                        display,
+                        max_texture_size,
+                        GSK_GPU_DEVICE_DEFAULT_TILE_SIZE,
+                        globals_alignment);
 
   self->version_string = gdk_gl_context_get_glsl_version_string (context);
   self->api = gdk_gl_context_get_api (context);
@@ -293,8 +306,9 @@ prepend_line_numbers (char *code)
 }
 
 static gboolean
-gsk_gl_device_check_shader_error (int      shader_id,
-                                  GError **error)
+gsk_gl_device_check_shader_error (const char  *name,
+                                  int          shader_id,
+                                  GError     **error)
 {
   GLint status;
   GLint log_len;
@@ -320,13 +334,14 @@ gsk_gl_device_check_shader_error (int      shader_id,
   g_set_error (error,
                GDK_GL_ERROR,
                GDK_GL_ERROR_COMPILATION_FAILED,
-               "Compilation failure in shader.\n"
+               "Compilation failure in shader %s.\n"
                "Source Code:\n"
                "%s\n"
                "\n"
                "Error Message:\n"
                "%s\n"
                "\n",
+               name,
                code,
                log);
 
@@ -366,13 +381,13 @@ print_shader_info (const char *prefix,
 }
 
 static GLuint
-gsk_gl_device_load_shader (GskGLDevice      *self,
-                           const char       *program_name,
-                           GLenum            shader_type,
-                           guint32           variation,
-                           GskGpuShaderClip  clip,
-                           guint             n_external_textures,
-                           GError          **error)
+gsk_gl_device_load_shader (GskGLDevice       *self,
+                           const char        *program_name,
+                           GLenum             shader_type,
+                           GskGpuShaderFlags  flags,
+                           GskGpuColorStates  color_states,
+                           guint32            variation,
+                           GError           **error)
 {
   GString *preamble;
   char *resource_name;
@@ -385,21 +400,23 @@ gsk_gl_device_load_shader (GskGLDevice      *self,
   g_string_append (preamble, "\n");
   if (self->api == GDK_GL_API_GLES)
     {
-      if (n_external_textures > 0)
+      if (gsk_gpu_shader_flags_has_external_textures (flags))
         {
           g_string_append (preamble, "#extension GL_OES_EGL_image_external_essl3 : require\n");
           g_string_append (preamble, "#extension GL_OES_EGL_image_external : require\n");
         }
       g_string_append (preamble, "#define GSK_GLES 1\n");
-      g_assert (3 * n_external_textures <= 16);
     }
   else
     {
-      g_assert (n_external_textures == 0);
+      g_assert (!gsk_gpu_shader_flags_has_external_textures (flags));
     }
 
-  g_string_append_printf (preamble, "#define N_TEXTURES %u\n", 16 - 3 * n_external_textures);
-  g_string_append_printf (preamble, "#define N_EXTERNAL_TEXTURES %u\n", n_external_textures);
+  /* work with AMD's compiler, too */
+  if (gsk_gpu_shader_flags_has_external_texture0 (flags))
+    g_string_append (preamble, "#define GSK_TEXTURE0_IS_EXTERNAL 1\n");
+  if (gsk_gpu_shader_flags_has_external_texture1 (flags))
+    g_string_append (preamble, "#define GSK_TEXTURE1_IS_EXTERNAL 1\n");
 
   switch (shader_type)
     {
@@ -416,24 +433,10 @@ gsk_gl_device_load_shader (GskGLDevice      *self,
         return 0;
     }
 
+  g_string_append_printf (preamble, "#define GSK_FLAGS %uu\n", flags);
+  g_string_append_printf (preamble, "#define GSK_COLOR_STATES %uu\n", color_states);
   g_string_append_printf (preamble, "#define GSK_VARIATION %uu\n", variation);
 
-  switch (clip)
-  {
-    case GSK_GPU_SHADER_CLIP_NONE:
-      g_string_append (preamble, "#define GSK_SHADER_CLIP GSK_GPU_SHADER_CLIP_NONE\n");
-      break;
-    case GSK_GPU_SHADER_CLIP_RECT:
-      g_string_append (preamble, "#define GSK_SHADER_CLIP GSK_GPU_SHADER_CLIP_RECT\n");
-      break;
-    case GSK_GPU_SHADER_CLIP_ROUNDED:
-      g_string_append (preamble, "#define GSK_SHADER_CLIP GSK_GPU_SHADER_CLIP_ROUNDED\n");
-      break;
-    default:
-      g_assert_not_reached ();
-      break;
-  }
-
   resource_name = g_strconcat ("/org/gtk/libgsk/shaders/gl/", program_name, ".glsl", NULL);
   bytes = g_resources_lookup_data (resource_name, 0, error);
   g_free (resource_name);
@@ -457,7 +460,7 @@ gsk_gl_device_load_shader (GskGLDevice      *self,
 
   print_shader_info (shader_type == GL_FRAGMENT_SHADER ? "fragment" : "vertex", shader_id, program_name);
 
-  if (!gsk_gl_device_check_shader_error (shader_id, error))
+  if (!gsk_gl_device_check_shader_error (program_name, shader_id, error))
     {
       glDeleteShader (shader_id);
       return 0;
@@ -469,20 +472,20 @@ gsk_gl_device_load_shader (GskGLDevice      *self,
 static GLuint
 gsk_gl_device_load_program (GskGLDevice               *self,
                             const GskGpuShaderOpClass *op_class,
+                            GskGpuShaderFlags          flags,
+                            GskGpuColorStates          color_states,
                             guint32                    variation,
-                            GskGpuShaderClip           clip,
-                            guint                      n_external_textures,
                             GError                   **error)
 {
   G_GNUC_UNUSED gint64 begin_time = GDK_PROFILER_CURRENT_TIME;
   GLuint vertex_shader_id, fragment_shader_id, program_id;
   GLint link_status;
 
-  vertex_shader_id = gsk_gl_device_load_shader (self, op_class->shader_name, GL_VERTEX_SHADER, variation, clip, n_external_textures, error);
+  vertex_shader_id = gsk_gl_device_load_shader (self, op_class->shader_name, GL_VERTEX_SHADER, flags, color_states, variation, error);
   if (vertex_shader_id == 0)
     return 0;
 
-  fragment_shader_id = gsk_gl_device_load_shader (self, op_class->shader_name, GL_FRAGMENT_SHADER, variation, clip, n_external_textures, error);
+  fragment_shader_id = gsk_gl_device_load_shader (self, op_class->shader_name, GL_FRAGMENT_SHADER, flags, color_states, variation, error);
   if (fragment_shader_id == 0)
     return 0;
 
@@ -540,19 +543,18 @@ gsk_gl_device_load_program (GskGLDevice               *self,
 void
 gsk_gl_device_use_program (GskGLDevice               *self,
                            const GskGpuShaderOpClass *op_class,
-                           guint32                    variation,
-                           GskGpuShaderClip           clip,
-                           guint                      n_external_textures)
+                           GskGpuShaderFlags          flags,
+                           GskGpuColorStates          color_states,
+                           guint32                    variation)
 {
   GError *error = NULL;
   GLuint program_id;
   GLProgramKey key = {
     .op_class = op_class,
+    .flags = flags,
+    .color_states = color_states,
     .variation = variation,
-    .clip = clip,
-    .n_external_textures = n_external_textures
   };
-  guint i, n_textures;
 
   program_id = GPOINTER_TO_UINT (g_hash_table_lookup (self->gl_programs, &key));
   if (program_id)
@@ -561,7 +563,7 @@ gsk_gl_device_use_program (GskGLDevice               *self,
       return;
     }
 
-  program_id = gsk_gl_device_load_program (self, op_class, variation, clip, n_external_textures, &error);
+  program_id = gsk_gl_device_load_program (self, op_class, flags, color_states, variation, &error);
   if (program_id == 0)
     {
       g_critical ("Failed to load shader program: %s", error->message);
@@ -569,25 +571,13 @@ gsk_gl_device_use_program (GskGLDevice               *self,
       return;
     }
   
-  g_hash_table_insert (self->gl_programs, g_memdup (&key, sizeof (GLProgramKey)), GUINT_TO_POINTER (program_id));
+  g_hash_table_insert (self->gl_programs, g_memdup2 (&key, sizeof (GLProgramKey)), GUINT_TO_POINTER (program_id));
 
   glUseProgram (program_id);
 
-  n_textures = 16 - 3 * n_external_textures;
-
-  for (i = 0; i < n_external_textures; i++)
-    {
-      char *name = g_strdup_printf ("external_textures[%u]", i);
-      glUniform1i (glGetUniformLocation (program_id, name), n_textures + 3 * i);
-      g_free (name);
-    }
-
-  for (i = 0; i < n_textures; i++)
-    {
-      char *name = g_strdup_printf ("textures[%u]", i);
-      glUniform1i (glGetUniformLocation (program_id, name), i);
-      g_free (name);
-    }
+  /* space by 3 because external textures may need 3 texture units */
+  glUniform1i (glGetUniformLocation (program_id, "GSK_TEXTURE0"), 0);
+  glUniform1i (glGetUniformLocation (program_id, "GSK_TEXTURE1"), 3);
 }
 
 GLuint
@@ -599,10 +589,25 @@ gsk_gl_device_get_sampler_id (GskGLDevice   *self,
   return self->sampler_ids[sampler];
 }
 
+static gboolean
+gsk_gl_swizzle_is_framebuffer_compatible (GLint swizzle[4])
+{
+  if (swizzle[0] != GL_RED ||
+      swizzle[1] != GL_GREEN ||
+      swizzle[2] != GL_BLUE ||
+      /* We accept GL_ONE here because that is used for RGBX channels and in that case
+       * this value is ignored */
+      (swizzle[3] != GL_ALPHA && swizzle[3] != GL_ONE))
+    return FALSE;
+
+  return TRUE;
+}
+
 static gboolean
 gsk_gl_device_get_format_flags (GskGLDevice      *self,
                                 GdkGLContext     *context,
                                 GdkMemoryFormat   format,
+                                GLint             swizzle[4],
                                 GskGpuImageFlags *out_flags)
 {
   GdkGLMemoryFlags gl_flags;
@@ -613,7 +618,7 @@ gsk_gl_device_get_format_flags (GskGLDevice      *self,
   if (!(gl_flags & GDK_GL_FORMAT_USABLE))
     return FALSE;
 
-  if (gl_flags & GDK_GL_FORMAT_RENDERABLE)
+  if ((gl_flags & GDK_GL_FORMAT_RENDERABLE) && gsk_gl_swizzle_is_framebuffer_compatible (swizzle))
     *out_flags |= GSK_GPU_IMAGE_RENDERABLE;
   else if (gdk_gl_context_get_use_es (context))
     *out_flags |= GSK_GPU_IMAGE_NO_BLIT;
@@ -635,6 +640,7 @@ gsk_gl_device_find_gl_format (GskGLDevice      *self,
                               GdkMemoryFormat  *out_format,
                               GskGpuImageFlags *out_flags,
                               GLint            *out_gl_internal_format,
+                              GLint            *out_gl_internal_srgb_format,
                               GLenum           *out_gl_format,
                               GLenum           *out_gl_type,
                               GLint             out_swizzle[4])
@@ -646,17 +652,18 @@ gsk_gl_device_find_gl_format (GskGLDevice      *self,
   gsize i;
 
   /* First, try the actual format */
-  if (gsk_gl_device_get_format_flags (self, context, format, &flags) &&
+  gdk_memory_format_gl_format (format,
+                               gdk_gl_context_get_use_es (context),
+                               out_gl_internal_format,
+                               out_gl_internal_srgb_format,
+                               out_gl_format,
+                               out_gl_type,
+                               out_swizzle);
+  if (gsk_gl_device_get_format_flags (self, context, format, out_swizzle, &flags) &&
       ((flags & required_flags) == required_flags))
     {
       *out_format = format;
       *out_flags = flags;
-      gdk_memory_format_gl_format (format,
-                                   gdk_gl_context_get_use_es (context),
-                                   out_gl_internal_format,
-                                   out_gl_format,
-                                   out_gl_type,
-                                   out_swizzle);
       return;
     }
 
@@ -665,10 +672,11 @@ gsk_gl_device_find_gl_format (GskGLDevice      *self,
                                         gdk_gl_context_get_use_es (context),
                                         &alt_format,
                                         out_gl_internal_format,
+                                        out_gl_internal_srgb_format,
                                         out_gl_format,
                                         out_gl_type,
                                         out_swizzle) &&
-      gsk_gl_device_get_format_flags (self, context, alt_format, &flags) &&
+      gsk_gl_device_get_format_flags (self, context, alt_format, out_swizzle, &flags) &&
       ((flags & required_flags) == required_flags))
     {
       *out_format = format;
@@ -680,17 +688,18 @@ gsk_gl_device_find_gl_format (GskGLDevice      *self,
   fallbacks = gdk_memory_format_get_fallbacks (format);
   for (i = 0; fallbacks[i] != -1; i++)
     {
-      if (gsk_gl_device_get_format_flags (self, context, fallbacks[i], &flags) &&
+      gdk_memory_format_gl_format (fallbacks[i],
+                                   gdk_gl_context_get_use_es (context),
+                                   out_gl_internal_format,
+                                   out_gl_internal_srgb_format,
+                                   out_gl_format,
+                                   out_gl_type,
+                                   out_swizzle);
+      if (gsk_gl_device_get_format_flags (self, context, fallbacks[i], out_swizzle, &flags) &&
           ((flags & required_flags) == required_flags))
         {
           *out_format = fallbacks[i];
           *out_flags = flags;
-          gdk_memory_format_gl_format (fallbacks[i],
-                                       gdk_gl_context_get_use_es (context),
-                                       out_gl_internal_format,
-                                       out_gl_format,
-                                       out_gl_type,
-                                       out_swizzle);
           return;
         }
     }
