@@ -180,33 +180,6 @@ gsk_gpu_renderer_dmabuf_downloader_init (GdkDmabufDownloaderInterface *iface)
   iface->download = gsk_gpu_renderer_dmabuf_downloader_download;
 }
 
-static cairo_region_t *
-get_render_region (GskGpuRenderer *self)
-{
-  GskGpuRendererPrivate *priv = gsk_gpu_renderer_get_instance_private (self);
-  const cairo_region_t *damage;
-  cairo_region_t *scaled_damage;
-  double scale;
-
-  scale = gdk_surface_get_scale (gdk_draw_context_get_surface (priv->context));
-
-  damage = gdk_draw_context_get_frame_region (priv->context);
-  scaled_damage = cairo_region_create ();
-  for (int i = 0; i < cairo_region_num_rectangles (damage); i++)
-    {
-      cairo_rectangle_int_t rect;
-      cairo_region_get_rectangle (damage, i, &rect);
-      cairo_region_union_rectangle (scaled_damage, &(cairo_rectangle_int_t) {
-                                      .x = (int) floor (rect.x * scale),
-                                      .y = (int) floor (rect.y * scale),
-                                      .width = (int) ceil ((rect.x + rect.width) * scale) - floor (rect.x * scale),
-                                      .height = (int) ceil ((rect.y + rect.height) * scale) - floor (rect.y * scale),
-                                    });
-    }
-
-  return scaled_damage;
-}
-
 static gboolean
 gsk_gpu_renderer_realize (GskRenderer  *renderer,
                           GdkDisplay   *display,
@@ -303,12 +276,16 @@ gsk_gpu_renderer_fallback_render_texture (GskGpuRenderer        *self,
     {
       for (x = 0; x < width; x += image_width)
         {
+          gsize tile_width, tile_height;
+
+          tile_width = MIN (image_width, width - x);
+          tile_height = MIN (image_height, height - y);
           texture = NULL;
+
           if (image == NULL)
             image = gsk_gpu_device_create_download_image (priv->device,
                                                           depth,
-                                                          MIN (image_width, width - x),
-                                                          MIN (image_height, height - y));
+                                                          tile_width, tile_height);
 
           if (gsk_gpu_image_get_flags (image) & GSK_GPU_IMAGE_SRGB)
             color_state = GDK_COLOR_STATE_SRGB_LINEAR;
@@ -317,8 +294,7 @@ gsk_gpu_renderer_fallback_render_texture (GskGpuRenderer        *self,
 
           clip_region = cairo_region_create_rectangle (&(cairo_rectangle_int_t) {
                                                            0, 0,
-                                                           gsk_gpu_image_get_width (image),
-                                                           gsk_gpu_image_get_height (image)
+                                                           tile_width, tile_height
                                                        });
           frame = gsk_gpu_renderer_get_frame (self);
           gsk_gpu_frame_render (frame,
@@ -329,8 +305,8 @@ gsk_gpu_renderer_fallback_render_texture (GskGpuRenderer        *self,
                                 root,
                                 &GRAPHENE_RECT_INIT (rounded_viewport->origin.x + x,
                                                      rounded_viewport->origin.y + y,
-                                                     image_width,
-                                                     image_height),
+                                                     tile_width,
+                                                     tile_height),
                                 &texture);
           gsk_gpu_frame_wait (frame);
 
@@ -347,6 +323,7 @@ gsk_gpu_renderer_fallback_render_texture (GskGpuRenderer        *self,
 
           /* Let's GC like a madman, we draw oversized stuff and don't want to OOM */
           gsk_gpu_device_maybe_gc (priv->device);
+          gsk_gpu_renderer_make_current (self);
         }
     }
 
@@ -457,7 +434,7 @@ gsk_gpu_renderer_render (GskRenderer          *renderer,
 
   backbuffer = GSK_GPU_RENDERER_GET_CLASS (self)->get_backbuffer (self);
 
-  render_region = get_render_region (self);
+  render_region = cairo_region_copy (gdk_draw_context_get_render_region (priv->context));
 
   gsk_gpu_frame_render (frame,
                         g_get_monotonic_time (),
