@@ -37,6 +37,7 @@
 #include "gtkbox.h"
 #include "gtklabel.h"
 #include "gtkbutton.h"
+#include "gtkimage.h"
 #include "gtkenums.h"
 #include "deprecated/gtkdialog.h"
 #include "gtkrevealer.h"
@@ -46,14 +47,19 @@
 #include "gtkbinlayout.h"
 #include "gtkgestureclick.h"
 
+#include <glib/gi18n-lib.h>
+
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 
 /**
  * GtkInfoBar:
  *
- * `GtkInfoBar` can be show messages to the user without a dialog.
+ * `GtkInfoBar` can be used to show messages to the user without a dialog.
  *
- * ![An example GtkInfoBar](info-bar.png)
+ * <picture>
+ *   <source srcset="info-bar-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkInfoBar" src="info-bar.png">
+ * </picture>
  *
  * It is often temporarily shown at the top or bottom of a document.
  * In contrast to [class@Gtk.Dialog], which has an action area at the
@@ -352,7 +358,7 @@ gtk_info_bar_class_init (GtkInfoBarClass *klass)
   klass->close = gtk_info_bar_close;
 
   /**
-   * GtkInfoBar:message-type: (attributes org.gtk.Property.get=gtk_info_bar_get_message_type org.gtk.Property.set=gtk_info_bar_set_message_type)
+   * GtkInfoBar:message-type:
    *
    * The type of the message.
    *
@@ -365,7 +371,7 @@ gtk_info_bar_class_init (GtkInfoBarClass *klass)
                        GTK_PARAM_READWRITE|G_PARAM_CONSTRUCT|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkInfoBar:show-close-button: (attributes org.gtk.Property.get=gtk_info_bar_get_show_close_button org.gtk.Property.set=gtk_info_bar_set_show_close_button)
+   * GtkInfoBar:show-close-button:
    *
    * Whether to include a standard close button.
    */
@@ -375,7 +381,7 @@ gtk_info_bar_class_init (GtkInfoBarClass *klass)
                           GTK_PARAM_READWRITE|G_PARAM_CONSTRUCT|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkInfoBar:revealed: (attributes org.gtk.Property.get=gtk_info_bar_get_revealed org.gtk.Property.set=gtk_info_bar_set_revealed)
+   * GtkInfoBar:revealed:
    *
    * Whether the info bar shows its contents.
    */
@@ -456,6 +462,7 @@ gtk_info_bar_init (GtkInfoBar *info_bar)
   GtkWidget *widget = GTK_WIDGET (info_bar);
   GtkWidget *main_box;
   GtkGesture *gesture;
+  GtkWidget *image;
 
   /* message-type is a CONSTRUCT property, so we init to a value
    * different from its default to trigger its property setter
@@ -478,11 +485,23 @@ gtk_info_bar_init (GtkInfoBar *info_bar)
   gtk_widget_set_valign (info_bar->action_area, GTK_ALIGN_CENTER);
   gtk_box_append (GTK_BOX (main_box), info_bar->action_area);
 
-  info_bar->close_button = gtk_button_new_from_icon_name ("window-close-symbolic");
+  info_bar->close_button = gtk_button_new ();
+  /* The icon is not relevant for accessibility purposes */
+  image = g_object_new (GTK_TYPE_IMAGE,
+                        "accessible-role", GTK_ACCESSIBLE_ROLE_PRESENTATION,
+                        "icon-name", "window-close-symbolic",
+                        "use-fallback", TRUE,
+                        NULL);
+  gtk_button_set_child (GTK_BUTTON (info_bar->close_button), image);
   gtk_widget_hide (info_bar->close_button);
   gtk_widget_set_valign (info_bar->close_button, GTK_ALIGN_CENTER);
   gtk_widget_add_css_class (info_bar->close_button, "close");
   gtk_box_append (GTK_BOX (main_box), info_bar->close_button);
+  gtk_accessible_update_property (GTK_ACCESSIBLE (info_bar->close_button),
+                                  GTK_ACCESSIBLE_PROPERTY_LABEL, _("Close"),
+                                  GTK_ACCESSIBLE_PROPERTY_DESCRIPTION, _("Close the infobar"),
+                                  -1);
+
   g_signal_connect (info_bar->close_button, "clicked",
                     G_CALLBACK (close_button_clicked_cb), info_bar);
 
@@ -1088,7 +1107,7 @@ gtk_info_bar_buildable_add_child (GtkBuildable *buildable,
 }
 
 /**
- * gtk_info_bar_set_message_type: (attributes org.gtk.Method.set_property=message-type)
+ * gtk_info_bar_set_message_type:
  * @info_bar: a `GtkInfoBar`
  * @message_type: a `GtkMessageType`
  *
@@ -1129,7 +1148,7 @@ gtk_info_bar_set_message_type (GtkInfoBar     *info_bar,
 }
 
 /**
- * gtk_info_bar_get_message_type: (attributes org.gtk.Method.get_property=message-type)
+ * gtk_info_bar_get_message_type:
  * @info_bar: a `GtkInfoBar`
  *
  * Returns the message type of the message area.
@@ -1148,7 +1167,7 @@ gtk_info_bar_get_message_type (GtkInfoBar *info_bar)
 
 
 /**
- * gtk_info_bar_set_show_close_button: (attributes org.gtk.Method.set_property=show-close-button)
+ * gtk_info_bar_set_show_close_button:
  * @info_bar: a `GtkInfoBar`
  * @setting: %TRUE to include a close button
  *
@@ -1172,7 +1191,7 @@ gtk_info_bar_set_show_close_button (GtkInfoBar *info_bar,
 }
 
 /**
- * gtk_info_bar_get_show_close_button: (attributes org.gtk.Method.get_property=show-close-button)
+ * gtk_info_bar_get_show_close_button:
  * @info_bar: a `GtkInfoBar`
  *
  * Returns whether the widget will display a standard close button.
@@ -1190,7 +1209,7 @@ gtk_info_bar_get_show_close_button (GtkInfoBar *info_bar)
 }
 
 /**
- * gtk_info_bar_set_revealed: (attributes org.gtk.Method.set_property=revealed)
+ * gtk_info_bar_set_revealed:
  * @info_bar: a `GtkInfoBar`
  * @revealed: The new value of the property
  *
@@ -1219,7 +1238,7 @@ gtk_info_bar_set_revealed (GtkInfoBar *info_bar,
 }
 
 /**
- * gtk_info_bar_get_revealed: (attributes org.gtk.Method.get_property=revealed)
+ * gtk_info_bar_get_revealed:
  * @info_bar: a `GtkInfoBar`
  *
  * Returns whether the info bar is currently revealed.
