@@ -23,9 +23,9 @@
 #include "gtkcolumnviewsorterprivate.h"
 
 #include "gtkcolumnviewprivate.h"
+#include "gtkcolumnviewrowwidgetprivate.h"
 #include "gtkcolumnviewtitleprivate.h"
 #include "gtklistbaseprivate.h"
-#include "gtklistitemwidgetprivate.h"
 #include "gtkmain.h"
 #include "gtkprivate.h"
 #include "gtkrbtreeprivate.h"
@@ -36,7 +36,7 @@
 /**
  * GtkColumnViewColumn:
  *
- * `GtkColumnViewColumn` represents the columns being added to `GtkColumnView`.
+ * Represents the columns in a `GtkColumnView`.
  *
  * The main ingredient for a `GtkColumnViewColumn` is the `GtkListItemFactory`
  * that tells the columnview how to create cells for this column from items in
@@ -56,6 +56,7 @@ struct _GtkColumnViewColumn
 
   GtkListItemFactory *factory;
   char *title;
+  char *id;
   GtkSorter *sorter;
 
   /* data for the view */
@@ -77,7 +78,7 @@ struct _GtkColumnViewColumn
   GMenuModel *menu;
 
   /* This list isn't sorted - this is just caching for performance */
-  GtkColumnViewCell *first_cell; /* no reference, just caching */
+  GtkColumnViewCellWidget *first_cell; /* no reference, just caching */
 };
 
 struct _GtkColumnViewColumnClass
@@ -97,6 +98,7 @@ enum
   PROP_RESIZABLE,
   PROP_EXPAND,
   PROP_FIXED_WIDTH,
+  PROP_ID,
 
   N_PROPS
 };
@@ -117,6 +119,7 @@ gtk_column_view_column_dispose (GObject *object)
   g_clear_object (&self->sorter);
   g_clear_pointer (&self->title, g_free);
   g_clear_object (&self->menu);
+  g_clear_pointer (&self->id, g_free);
 
   G_OBJECT_CLASS (gtk_column_view_column_parent_class)->dispose (object);
 }
@@ -167,6 +170,10 @@ gtk_column_view_column_get_property (GObject    *object,
       g_value_set_int (value, self->fixed_width);
       break;
 
+    case PROP_ID:
+      g_value_set_string (value, self->id);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -215,6 +222,10 @@ gtk_column_view_column_set_property (GObject      *object,
       gtk_column_view_column_set_fixed_width (self, g_value_get_int (value));
       break;
 
+    case PROP_ID:
+      gtk_column_view_column_set_id (self, g_value_get_string (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -231,7 +242,7 @@ gtk_column_view_column_class_init (GtkColumnViewColumnClass *klass)
   gobject_class->set_property = gtk_column_view_column_set_property;
 
   /**
-   * GtkColumnViewColumn:column-view: (attributes org.gtk.Property.get=gtk_column_view_column_get_column_view)
+   * GtkColumnViewColumn:column-view:
    *
    * The `GtkColumnView` this column is a part of.
    */
@@ -241,9 +252,11 @@ gtk_column_view_column_class_init (GtkColumnViewColumnClass *klass)
                          G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkColumnViewColumn:factory: (attributes org.gtk.Property.get=gtk_column_view_column_get_factory org.gtk.Property.set=gtk_column_view_column_set_factory)
+   * GtkColumnViewColumn:factory:
    *
    * Factory for populating list items.
+   *
+   * The factory must be for configuring [class@Gtk.ColumnViewCell] objects.
    */
   properties[PROP_FACTORY] =
     g_param_spec_object ("factory", NULL, NULL,
@@ -251,7 +264,7 @@ gtk_column_view_column_class_init (GtkColumnViewColumnClass *klass)
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkColumnViewColumn:title: (attributes org.gtk.Property.get=gtk_column_view_column_get_title org.gtk.Property.set=gtk_column_view_column_set_title)
+   * GtkColumnViewColumn:title:
    *
    * Title displayed in the header.
    */
@@ -261,7 +274,7 @@ gtk_column_view_column_class_init (GtkColumnViewColumnClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkColumnViewColumn:sorter: (attributes org.gtk.Property.get=gtk_column_view_column_get_sorter org.gtk.Property.set=gtk_column_view_column_set_sorter)
+   * GtkColumnViewColumn:sorter:
    *
    * Sorter for sorting items according to this column.
    */
@@ -271,7 +284,7 @@ gtk_column_view_column_class_init (GtkColumnViewColumnClass *klass)
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkColumnViewColumn:visible: (attributes org.gtk.Property.get=gtk_column_view_column_get_visible org.gtk.Property.set=gtk_column_view_column_set_visible)
+   * GtkColumnViewColumn:visible:
    *
    * Whether this column is visible.
    */
@@ -281,7 +294,7 @@ gtk_column_view_column_class_init (GtkColumnViewColumnClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkColumnViewColumn:header-menu: (attributes org.gtk.Property.get=gtk_column_view_column_get_header_menu org.gtk.Property.set=gtk_column_view_column_set_header_menu)
+   * GtkColumnViewColumn:header-menu:
    *
    * Menu model used to create the context menu for the column header.
    */
@@ -291,7 +304,7 @@ gtk_column_view_column_class_init (GtkColumnViewColumnClass *klass)
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkColumnViewColumn:resizable: (attributes org.gtk.Property.get=gtk_column_view_column_get_resizable org.gtk.Property.set=gtk_column_view_column_set_resizable)
+   * GtkColumnViewColumn:resizable:
    *
    * Whether this column is resizable.
    */
@@ -301,7 +314,7 @@ gtk_column_view_column_class_init (GtkColumnViewColumnClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkColumnViewColumn:expand: (attributes org.gtk.Property.get=gtk_column_view_column_get_expand org.gtk.Property.set=gtk_column_view_column_set_expand)
+   * GtkColumnViewColumn:expand:
    *
    * Column gets share of extra width allocated to the view.
    */
@@ -311,7 +324,7 @@ gtk_column_view_column_class_init (GtkColumnViewColumnClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkColumnViewColumn:fixed-width: (attributes org.gtk.Property.get=gtk_column_view_column_get_fixed_width org.gtk.Property.set=gtk_column_view_column_set_fixed_width)
+   * GtkColumnViewColumn:fixed-width:
    *
    * If not -1, this is the width that the column is allocated,
    * regardless of the size of its content.
@@ -321,6 +334,24 @@ gtk_column_view_column_class_init (GtkColumnViewColumnClass *klass)
                       -1, G_MAXINT, -1,
                       G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * GtkColumnViewColumn:id:
+   *
+   * An ID for the column.
+   *
+   * GTK is not currently using the ID for anything, but
+   * it can be used by applications when saving column view
+   * configurations.
+   *
+   * It is up to applications to ensure uniqueness of IDs.
+   *
+   * Since: 4.10
+   */
+  properties[PROP_ID] =
+    g_param_spec_string ("id", NULL, NULL,
+                          NULL,
+                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (gobject_class, N_PROPS, properties);
 }
 
@@ -372,7 +403,7 @@ gtk_column_view_column_new (const char         *title,
   return result;
 }
 
-GtkColumnViewCell *
+GtkColumnViewCellWidget *
 gtk_column_view_column_get_first_cell (GtkColumnViewColumn *self)
 {
   return self->first_cell;
@@ -380,7 +411,7 @@ gtk_column_view_column_get_first_cell (GtkColumnViewColumn *self)
 
 void
 gtk_column_view_column_add_cell (GtkColumnViewColumn *self,
-                                 GtkColumnViewCell   *cell)
+                                 GtkColumnViewCellWidget   *cell)
 {
   self->first_cell = cell;
 
@@ -390,10 +421,10 @@ gtk_column_view_column_add_cell (GtkColumnViewColumn *self,
 
 void
 gtk_column_view_column_remove_cell (GtkColumnViewColumn *self,
-                                    GtkColumnViewCell   *cell)
+                                    GtkColumnViewCellWidget   *cell)
 {
   if (cell == self->first_cell)
-    self->first_cell = gtk_column_view_cell_get_next (cell);
+    self->first_cell = gtk_column_view_cell_widget_get_next (cell);
 
   gtk_column_view_column_queue_resize (self);
   gtk_widget_queue_resize (GTK_WIDGET (cell));
@@ -402,7 +433,7 @@ gtk_column_view_column_remove_cell (GtkColumnViewColumn *self,
 void
 gtk_column_view_column_queue_resize (GtkColumnViewColumn *self)
 {
-  GtkColumnViewCell *cell;
+  GtkColumnViewCellWidget *cell;
 
   if (self->minimum_size_request < 0)
     return;
@@ -413,7 +444,7 @@ gtk_column_view_column_queue_resize (GtkColumnViewColumn *self)
   if (self->header)
     gtk_widget_queue_resize (self->header);
 
-  for (cell = self->first_cell; cell; cell = gtk_column_view_cell_get_next (cell))
+  for (cell = self->first_cell; cell; cell = gtk_column_view_cell_widget_get_next (cell))
     {
       gtk_widget_queue_resize (GTK_WIDGET (cell));
     }
@@ -432,7 +463,7 @@ gtk_column_view_column_measure (GtkColumnViewColumn *self,
 
   if (self->minimum_size_request < 0)
     {
-      GtkColumnViewCell *cell;
+      GtkColumnViewCellWidget *cell;
       int min, nat, cell_min, cell_nat;
 
       if (self->header)
@@ -445,7 +476,7 @@ gtk_column_view_column_measure (GtkColumnViewColumn *self,
           nat = 0;
         }
 
-      for (cell = self->first_cell; cell; cell = gtk_column_view_cell_get_next (cell))
+      for (cell = self->first_cell; cell; cell = gtk_column_view_cell_widget_get_next (cell))
         {
           gtk_widget_measure (GTK_WIDGET (cell),
                               GTK_ORIENTATION_HORIZONTAL,
@@ -500,19 +531,18 @@ gtk_column_view_column_create_cells (GtkColumnViewColumn *self)
        row != NULL;
        row = gtk_widget_get_next_sibling (row))
     {
-      GtkListItemWidget *list_item;
+      GtkColumnViewRowWidget *list_item;
+      GtkListItemBase *base;
       GtkWidget *cell;
 
-      if (!gtk_widget_get_root (row))
-        continue;
-
-      list_item = GTK_LIST_ITEM_WIDGET (row);
-      cell = gtk_column_view_cell_new (self);
-      gtk_list_item_widget_add_child (list_item, cell);
-      gtk_list_item_widget_update (GTK_LIST_ITEM_WIDGET (cell),
-                                   gtk_list_item_widget_get_position (list_item),
-                                   gtk_list_item_widget_get_item (list_item),
-                                   gtk_list_item_widget_get_selected (list_item));
+      list_item = GTK_COLUMN_VIEW_ROW_WIDGET (row);
+      base = GTK_LIST_ITEM_BASE (row);
+      cell = gtk_column_view_cell_widget_new (self, gtk_column_view_is_inert (self->view));
+      gtk_column_view_row_widget_add_child (list_item, cell);
+      gtk_list_item_base_update (GTK_LIST_ITEM_BASE (cell),
+                                 gtk_list_item_base_get_position (base),
+                                 gtk_list_item_base_get_item (base),
+                                 gtk_list_item_base_get_selected (base));
     }
 }
 
@@ -520,7 +550,7 @@ static void
 gtk_column_view_column_remove_cells (GtkColumnViewColumn *self)
 {
   while (self->first_cell)
-    gtk_column_view_cell_remove (self->first_cell);
+    gtk_column_view_cell_widget_remove (self->first_cell);
 }
 
 static void
@@ -531,7 +561,7 @@ gtk_column_view_column_create_header (GtkColumnViewColumn *self)
 
   self->header = gtk_column_view_title_new (self);
   gtk_widget_set_visible (self->header, self->visible);
-  gtk_list_item_widget_add_child (gtk_column_view_get_header_widget (self->view),
+  gtk_column_view_row_widget_add_child (gtk_column_view_get_header_widget (self->view),
                                   self->header);
   gtk_column_view_column_queue_resize (self);
 }
@@ -542,7 +572,7 @@ gtk_column_view_column_remove_header (GtkColumnViewColumn *self)
   if (self->header == NULL)
     return;
 
-  gtk_list_item_widget_remove_child (gtk_column_view_get_header_widget (self->view),
+  gtk_column_view_row_widget_remove_child (gtk_column_view_get_header_widget (self->view),
                                      self->header);
   self->header = NULL;
   gtk_column_view_column_queue_resize (self);
@@ -551,7 +581,7 @@ gtk_column_view_column_remove_header (GtkColumnViewColumn *self)
 static void
 gtk_column_view_column_ensure_cells (GtkColumnViewColumn *self)
 {
-  if (self->view && gtk_widget_get_root (GTK_WIDGET (self->view)))
+  if (self->view && gtk_column_view_column_get_visible (self))
     gtk_column_view_column_create_cells (self);
   else
     gtk_column_view_column_remove_cells (self);
@@ -563,12 +593,12 @@ gtk_column_view_column_ensure_cells (GtkColumnViewColumn *self)
 }
 
 /**
- * gtk_column_view_column_get_column_view: (attributes org.gtk.Method.get_property=column-view)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_get_column_view:
+ * @self: a column
  *
  * Gets the column view that's currently displaying this column.
  *
- * If @self has not been added to a column view yet, %NULL is returned.
+ * If @self has not been added to a column view yet, `NULL` is returned.
  *
  * Returns: (nullable) (transfer none): The column view displaying @self.
  */
@@ -601,27 +631,27 @@ void
 gtk_column_view_column_set_position (GtkColumnViewColumn *self,
                                      guint                position)
 {
-  GtkColumnViewCell *cell;
+  GtkColumnViewCellWidget *cell;
 
-  gtk_list_item_widget_reorder_child (gtk_column_view_get_header_widget (self->view),
+  gtk_column_view_row_widget_reorder_child (gtk_column_view_get_header_widget (self->view),
                                       self->header,
                                       position);
 
-  for (cell = self->first_cell; cell; cell = gtk_column_view_cell_get_next (cell))
+  for (cell = self->first_cell; cell; cell = gtk_column_view_cell_widget_get_next (cell))
     {
-      GtkListItemWidget *list_item;
+      GtkColumnViewRowWidget *list_item;
 
-      list_item = GTK_LIST_ITEM_WIDGET (gtk_widget_get_parent (GTK_WIDGET (cell)));
-      gtk_list_item_widget_reorder_child (list_item, GTK_WIDGET (cell), position);
+      list_item = GTK_COLUMN_VIEW_ROW_WIDGET (gtk_widget_get_parent (GTK_WIDGET (cell)));
+      gtk_column_view_row_widget_reorder_child (list_item, GTK_WIDGET (cell), position);
     }
 }
 
 /**
- * gtk_column_view_column_get_factory: (attributes org.gtk.Method.get_property=factory)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_get_factory:
+ * @self: a column
  *
- * Gets the factory that's currently used to populate list items for
- * this column.
+ * Gets the factory that's currently used to populate list items
+ * for this column.
  *
  * Returns: (nullable) (transfer none): The factory in use
  **/
@@ -633,13 +663,36 @@ gtk_column_view_column_get_factory (GtkColumnViewColumn *self)
   return self->factory;
 }
 
+void
+gtk_column_view_column_update_factory (GtkColumnViewColumn *self,
+                                       gboolean             inert)
+{
+  GtkListItemFactory *factory;
+  GtkColumnViewCellWidget *cell;
+
+  if (self->factory == NULL)
+    return;
+
+  if (inert)
+    factory = NULL;
+  else
+    factory = self->factory;
+
+  for (cell = self->first_cell;
+       cell;
+       cell = gtk_column_view_cell_widget_get_next (cell))
+    {
+      gtk_list_factory_widget_set_factory (GTK_LIST_FACTORY_WIDGET (cell), factory);
+    }
+}
+
 /**
- * gtk_column_view_column_set_factory: (attributes org.gtk.Method.set_property=factory)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_set_factory:
+ * @self: a column
  * @factory: (nullable) (transfer none): the factory to use
  *
- * Sets the `GtkListItemFactory` to use for populating list items for this
- * column.
+ * Sets the `GtkListItemFactory` to use for populating list items
+ * for this column.
  */
 void
 gtk_column_view_column_set_factory (GtkColumnViewColumn *self,
@@ -648,15 +701,21 @@ gtk_column_view_column_set_factory (GtkColumnViewColumn *self,
   g_return_if_fail (GTK_IS_COLUMN_VIEW_COLUMN (self));
   g_return_if_fail (factory == NULL || GTK_LIST_ITEM_FACTORY (factory));
 
+  if (self->factory && !factory)
+    gtk_column_view_column_update_factory (self, TRUE);
+
   if (!g_set_object (&self->factory, factory))
     return;
 
+  if (self->view && !gtk_column_view_is_inert (self->view))
+    gtk_column_view_column_update_factory (self, FALSE);
+
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_FACTORY]);
 }
 
 /**
- * gtk_column_view_column_set_title: (attributes org.gtk.Method.set_property=title)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_set_title:
+ * @self: a column
  * @title: (nullable): Title to use for this column
  *
  * Sets the title of this column.
@@ -678,16 +737,16 @@ gtk_column_view_column_set_title (GtkColumnViewColumn *self,
   self->title = g_strdup (title);
 
   if (self->header)
-    gtk_column_view_title_update (GTK_COLUMN_VIEW_TITLE (self->header));
+    gtk_column_view_title_set_title (GTK_COLUMN_VIEW_TITLE (self->header), title);
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_TITLE]);
 }
 
 /**
- * gtk_column_view_column_get_title: (attributes org.gtk.Method.get_property=title)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_get_title:
+ * @self: a column
  *
- * Returns the title set with gtk_column_view_column_set_title().
+ * Returns the title set with [method@Gtk.ColumnViewColumn.set_title].
  *
  * Returns: (nullable): The column's title
  */
@@ -699,34 +758,23 @@ gtk_column_view_column_get_title (GtkColumnViewColumn *self)
   return self->title;
 }
 
-#if 0
-static void
-gtk_column_view_column_add_to_sorter (GtkColumnViewColumn *self)
-{
-  if (self->view == NULL)
-    return;
-  
-  gtk_column_view_sorter_add_column (GTK_COLUMN_VIEW_SORTER (gtk_column_view_get_sorter (self->view)), self);
-}
-#endif
-
 static void
 gtk_column_view_column_remove_from_sorter (GtkColumnViewColumn *self)
 {
   if (self->view == NULL)
     return;
-  
+
   gtk_column_view_sorter_remove_column (GTK_COLUMN_VIEW_SORTER (gtk_column_view_get_sorter (self->view)), self);
 }
 
 /**
- * gtk_column_view_column_set_sorter: (attributes org.gtk.Method.set_property=sorter)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_set_sorter:
+ * @self: a column
  * @sorter: (nullable): the `GtkSorter` to associate with @column
  *
  * Associates a sorter with the column.
  *
- * If @sorter is %NULL, the column will not let users change
+ * If @sorter is unset, the column will not let users change
  * the sorting by clicking on its header.
  *
  * This sorter can be made active by clicking on the column
@@ -748,14 +796,14 @@ gtk_column_view_column_set_sorter (GtkColumnViewColumn *self,
   gtk_column_view_column_remove_from_sorter (self);
 
   if (self->header)
-    gtk_column_view_title_update (GTK_COLUMN_VIEW_TITLE (self->header));
+    gtk_column_view_title_update_sort (GTK_COLUMN_VIEW_TITLE (self->header));
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SORTER]);
 }
 
 /**
- * gtk_column_view_column_get_sorter: (attributes org.gtk.Method.get_property=sorter)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_get_sorter:
+ * @self: a column
  *
  * Returns the sorter that is associated with the column.
  *
@@ -773,12 +821,12 @@ void
 gtk_column_view_column_notify_sort (GtkColumnViewColumn *self)
 {
   if (self->header)
-    gtk_column_view_title_update (GTK_COLUMN_VIEW_TITLE (self->header));
+    gtk_column_view_title_update_sort (GTK_COLUMN_VIEW_TITLE (self->header));
 }
 
 /**
- * gtk_column_view_column_set_visible: (attributes org.gtk.Method.set_property=visible)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_set_visible:
+ * @self: a column
  * @visible: whether this column should be visible
  *
  * Sets whether this column should be visible in views.
@@ -787,8 +835,6 @@ void
 gtk_column_view_column_set_visible (GtkColumnViewColumn *self,
                                     gboolean             visible)
 {
-  GtkColumnViewCell *cell;
-
   g_return_if_fail (GTK_IS_COLUMN_VIEW_COLUMN (self));
 
   if (self->visible == visible)
@@ -802,21 +848,18 @@ gtk_column_view_column_set_visible (GtkColumnViewColumn *self,
   if (self->header)
     gtk_widget_set_visible (GTK_WIDGET (self->header), visible);
 
-  for (cell = self->first_cell; cell; cell = gtk_column_view_cell_get_next (cell))
-    {
-      gtk_widget_set_visible (GTK_WIDGET (cell), visible);
-    }
+  gtk_column_view_column_ensure_cells (self);
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_VISIBLE]);
 }
 
 /**
- * gtk_column_view_column_get_visible: (attributes org.gtk.Method.get_property=visible)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_get_visible:
+ * @self: a column
  *
  * Returns whether this column is visible.
  *
- * Returns: %TRUE if this column is visible
+ * Returns: true if this column is visible
  */
 gboolean
 gtk_column_view_column_get_visible (GtkColumnViewColumn *self)
@@ -827,8 +870,8 @@ gtk_column_view_column_get_visible (GtkColumnViewColumn *self)
 }
 
 /**
- * gtk_column_view_column_set_header_menu: (attributes org.gtk.Method.set_property=header-menu)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_set_header_menu:
+ * @self: a column
  * @menu: (nullable): a `GMenuModel`
  *
  * Sets the menu model that is used to create the context menu
@@ -845,14 +888,14 @@ gtk_column_view_column_set_header_menu (GtkColumnViewColumn *self,
     return;
 
   if (self->header)
-    gtk_column_view_title_update (GTK_COLUMN_VIEW_TITLE (self->header));
+    gtk_column_view_title_set_menu (GTK_COLUMN_VIEW_TITLE (self->header), menu);
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_HEADER_MENU]);
 }
 
 /**
- * gtk_column_view_column_get_header_menu: (attributes org.gtk.Method.get_property=header-menu)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_get_header_menu:
+ * @self: a column
  *
  * Gets the menu model that is used to create the context menu
  * for the column header.
@@ -868,14 +911,14 @@ gtk_column_view_column_get_header_menu (GtkColumnViewColumn *self)
 }
 
 /**
- * gtk_column_view_column_set_expand: (attributes org.gtk.Method.set_property=expand)
- * @self: a `GtkColumnViewColumn`
- * @expand: %TRUE if this column should expand to fill available sace
+ * gtk_column_view_column_set_expand:
+ * @self: a column
+ * @expand: whether this column should expand to fill available space
  *
  * Sets the column to take available extra space.
  *
  * The extra space is shared equally amongst all columns that
- * have the expand set to %TRUE.
+ * have are set to expand.
  */
 void
 gtk_column_view_column_set_expand (GtkColumnViewColumn *self,
@@ -895,12 +938,12 @@ gtk_column_view_column_set_expand (GtkColumnViewColumn *self,
 }
 
 /**
- * gtk_column_view_column_get_expand: (attributes org.gtk.Method.get_property=expand)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_get_expand:
+ * @self: a column
  *
  * Returns whether this column should expand.
  *
- * Returns: %TRUE if this column expands
+ * Returns: true if this column expands
  */
 gboolean
 gtk_column_view_column_get_expand (GtkColumnViewColumn *self)
@@ -911,8 +954,8 @@ gtk_column_view_column_get_expand (GtkColumnViewColumn *self)
 }
 
 /**
- * gtk_column_view_column_set_resizable: (attributes org.gtk.Method.set_property=resizable)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_set_resizable:
+ * @self: a column
  * @resizable: whether this column should be resizable
  *
  * Sets whether this column should be resizable by dragging.
@@ -932,12 +975,12 @@ gtk_column_view_column_set_resizable (GtkColumnViewColumn *self,
 }
 
 /**
- * gtk_column_view_column_get_resizable: (attributes org.gtk.Method.get_property=resizable)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_get_resizable:
+ * @self: a column
  *
  * Returns whether this column is resizable.
  *
- * Returns: %TRUE if this column is resizable
+ * Returns: true if this column is resizable
  */
 gboolean
 gtk_column_view_column_get_resizable (GtkColumnViewColumn *self)
@@ -948,12 +991,13 @@ gtk_column_view_column_get_resizable (GtkColumnViewColumn *self)
 }
 
 /**
- * gtk_column_view_column_set_fixed_width: (attributes org.gtk.Method.set_property=fixed-width)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_set_fixed_width:
+ * @self: a column
  * @fixed_width: the new fixed width, or -1
  *
- * If @fixed_width is not -1, sets the fixed width of @column;
- * otherwise unsets it.
+ * Sets the fixed width of the column.
+ *
+ * If @fixed_width is -1, the fixed width of the column is unset.
  *
  * Setting a fixed width overrides the automatically calculated
  * width. Interactive resizing also sets the “fixed-width” property.
@@ -976,8 +1020,8 @@ gtk_column_view_column_set_fixed_width (GtkColumnViewColumn *self,
 }
 
 /**
- * gtk_column_view_column_get_fixed_width: (attributes org.gtk.Method.get_property=fixed-width)
- * @self: a `GtkColumnViewColumn`
+ * gtk_column_view_column_get_fixed_width:
+ * @self: a column
  *
  * Gets the fixed width of the column.
  *
@@ -1015,3 +1059,50 @@ gtk_column_view_column_get_header_allocation (GtkColumnViewColumn *self,
   if (size)
     *size = self->allocation_size;
 }
+
+/**
+ * gtk_column_view_column_set_id:
+ * @self: a column
+ * @id: (nullable): ID to use for this column
+ *
+ * Sets the id of this column.
+ *
+ * GTK makes no use of this, but applications can use it when
+ * storing column view configuration.
+ *
+ * It is up to callers to ensure uniqueness of IDs.
+ *
+ * Since: 4.10
+ */
+void
+gtk_column_view_column_set_id (GtkColumnViewColumn *self,
+                               const char          *id)
+{
+  g_return_if_fail (GTK_IS_COLUMN_VIEW_COLUMN (self));
+
+  if (g_strcmp0 (self->id, id) == 0)
+    return;
+
+  g_free (self->id);
+  self->id = g_strdup (id);
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_ID]);
+}
+
+/**
+ * gtk_column_view_column_get_id:
+ * @self: a column
+ *
+ * Returns the ID set with [method@Gtk.ColumnViewColumn.set_id].
+ *
+ * Returns: (nullable): The column's ID
+ *
+ * Since: 4.10
+ */
+const char *
+gtk_column_view_column_get_id (GtkColumnViewColumn *self)
+{
+  g_return_val_if_fail (GTK_IS_COLUMN_VIEW_COLUMN (self), NULL);
+
+  return self->id;
+}
