@@ -35,17 +35,54 @@ gtk_css_value_array_free (GtkCssValue *value)
   guint i;
 
   for (i = 0; i < value->n_values; i++)
-    _gtk_css_value_unref (value->values[i]);
+    gtk_css_value_unref (value->values[i]);
 
   g_free (value);
 }
 
 static GtkCssValue *
-gtk_css_value_array_compute (GtkCssValue      *value,
-                             guint             property_id,
-                             GtkStyleProvider *provider,
-                             GtkCssStyle      *style,
-                             GtkCssStyle      *parent_style)
+gtk_css_value_array_compute (GtkCssValue          *value,
+                             guint                 property_id,
+                             GtkCssComputeContext *context)
+{
+  GtkCssValue *result;
+  GtkCssValue *i_value;
+  guint i, j;
+  gboolean contains_current_color = FALSE;;
+
+  result = NULL;
+  for (i = 0; i < value->n_values; i++)
+    {
+      i_value =  gtk_css_value_compute (value->values[i], property_id, context);
+      contains_current_color |= gtk_css_value_contains_current_color (i_value);
+
+      if (result == NULL &&
+          i_value != value->values[i])
+        {
+          result = _gtk_css_array_value_new_from_array (value->values, value->n_values);
+          for (j = 0; j < i; j++)
+            gtk_css_value_ref (result->values[j]);
+        }
+
+      if (result != NULL)
+        result->values[i] = i_value;
+      else
+        gtk_css_value_unref (i_value);
+    }
+
+  if (result == NULL)
+    return gtk_css_value_ref (value);
+
+  result->is_computed = TRUE;
+  result->contains_current_color = contains_current_color;
+
+  return result;
+}
+
+static GtkCssValue *
+gtk_css_value_array_resolve (GtkCssValue          *value,
+                             GtkCssComputeContext *context,
+                             GtkCssValue          *current)
 {
   GtkCssValue *result;
   GtkCssValue *i_value;
@@ -54,24 +91,27 @@ gtk_css_value_array_compute (GtkCssValue      *value,
   result = NULL;
   for (i = 0; i < value->n_values; i++)
     {
-      i_value =  _gtk_css_value_compute (value->values[i], property_id, provider, style, parent_style);
+      i_value =  gtk_css_value_resolve (value->values[i], context, current);
 
       if (result == NULL &&
-	  i_value != value->values[i])
-	{
-	  result = _gtk_css_array_value_new_from_array (value->values, value->n_values);
-	  for (j = 0; j < i; j++)
-	    _gtk_css_value_ref (result->values[j]);
-	}
+          i_value != value->values[i])
+        {
+          result = _gtk_css_array_value_new_from_array (value->values, value->n_values);
+          for (j = 0; j < i; j++)
+            gtk_css_value_ref (result->values[j]);
+        }
 
       if (result != NULL)
-	result->values[i] = i_value;
+        result->values[i] = i_value;
       else
-	_gtk_css_value_unref (i_value);
+        gtk_css_value_unref (i_value);
     }
 
   if (result == NULL)
-    return _gtk_css_value_ref (value);
+    return gtk_css_value_ref (value);
+
+  result->is_computed = TRUE;
+  result->contains_current_color = FALSE;
 
   return result;
 }
@@ -87,8 +127,8 @@ gtk_css_value_array_equal (const GtkCssValue *value1,
 
   for (i = 0; i < value1->n_values; i++)
     {
-      if (!_gtk_css_value_equal (value1->values[i],
-                                 value2->values[i]))
+      if (!gtk_css_value_equal (value1->values[i],
+                                value2->values[i]))
         return FALSE;
     }
 
@@ -127,14 +167,14 @@ gtk_css_value_array_transition_repeat (GtkCssValue *start,
 
   for (i = 0; i < n; i++)
     {
-      transitions[i] = _gtk_css_value_transition (start->values[i % start->n_values],
-                                                  end->values[i % end->n_values],
-                                                  property_id,
-                                                  progress);
+      transitions[i] = gtk_css_value_transition (start->values[i % start->n_values],
+                                                 end->values[i % end->n_values],
+                                                 property_id,
+                                                 progress);
       if (transitions[i] == NULL)
         {
           while (i--)
-            _gtk_css_value_unref (transitions[i]);
+            gtk_css_value_unref (transitions[i]);
           return NULL;
         }
     }
@@ -168,14 +208,14 @@ gtk_css_value_array_transition_extend (GtkCssValue *start,
 
   for (i = 0; i < MIN (start->n_values, end->n_values); i++)
     {
-      transitions[i] = _gtk_css_value_transition (start->values[i],
-                                                  end->values[i],
-                                                  property_id,
-                                                  progress);
+      transitions[i] = gtk_css_value_transition (start->values[i],
+                                                 end->values[i],
+                                                 property_id,
+                                                 progress);
       if (transitions[i] == NULL)
         {
           while (i--)
-            _gtk_css_value_unref (transitions[i]);
+            gtk_css_value_unref (transitions[i]);
           return NULL;
         }
     }
@@ -188,28 +228,28 @@ gtk_css_value_array_transition_extend (GtkCssValue *start,
 
       for (; i < start->n_values; i++)
         {
-          transitions[i] = _gtk_css_value_transition (start->values[i],
-                                                      default_value,
-                                                      property_id,
-                                                      progress);
+          transitions[i] = gtk_css_value_transition (start->values[i],
+                                                     default_value,
+                                                     property_id,
+                                                     progress);
           if (transitions[i] == NULL)
             {
               while (i--)
-                _gtk_css_value_unref (transitions[i]);
+                gtk_css_value_unref (transitions[i]);
               return NULL;
             }
         }
 
       for (; i < end->n_values; i++)
         {
-          transitions[i] = _gtk_css_value_transition (default_value,
-                                                      end->values[i],
-                                                      property_id,
-                                                      progress);
+          transitions[i] = gtk_css_value_transition (default_value,
+                                                     end->values[i],
+                                                     property_id,
+                                                     progress);
           if (transitions[i] == NULL)
             {
               while (i--)
-                _gtk_css_value_unref (transitions[i]);
+                gtk_css_value_unref (transitions[i]);
               return NULL;
             }
         }
@@ -327,17 +367,17 @@ gtk_css_value_array_get_dynamic_value (GtkCssValue *value,
 	{
 	  result = _gtk_css_array_value_new_from_array (value->values, value->n_values);
 	  for (j = 0; j < i; j++)
-	    _gtk_css_value_ref (result->values[j]);
+	    gtk_css_value_ref (result->values[j]);
 	}
 
       if (result != NULL)
 	result->values[i] = i_value;
       else
-	_gtk_css_value_unref (i_value);
+	gtk_css_value_unref (i_value);
     }
 
   if (result == NULL)
-    return _gtk_css_value_ref (value);
+    return gtk_css_value_ref (value);
 
   return result;
 }
@@ -358,7 +398,7 @@ gtk_css_value_array_print (const GtkCssValue *value,
     {
       if (i > 0)
         g_string_append (string, ", ");
-      _gtk_css_value_print (value->values[i], string);
+      gtk_css_value_print (value->values[i], string);
     }
 }
 
@@ -366,6 +406,7 @@ static const GtkCssValueClass GTK_CSS_VALUE_ARRAY = {
   "GtkCssArrayValue",
   gtk_css_value_array_free,
   gtk_css_value_array_compute,
+  gtk_css_value_array_resolve,
   gtk_css_value_array_equal,
   gtk_css_value_array_transition,
   gtk_css_value_array_is_dynamic,
@@ -394,18 +435,26 @@ _gtk_css_array_value_new_from_array (GtkCssValue **values,
   if (n_values == 1)
     return values[0];
 
-  result = _gtk_css_value_alloc (&GTK_CSS_VALUE_ARRAY, sizeof (GtkCssValue) + sizeof (GtkCssValue *) * (n_values - 1));
+  result = gtk_css_value_alloc (&GTK_CSS_VALUE_ARRAY, sizeof (GtkCssValue) + sizeof (GtkCssValue *) * (n_values - 1));
   result->n_values = n_values;
   memcpy (&result->values[0], values, sizeof (GtkCssValue *) * n_values);
 
   result->is_computed = TRUE;
+  result->contains_variables = FALSE;
+  result->contains_current_color = FALSE;
   for (i = 0; i < n_values; i ++)
     {
       if (!gtk_css_value_is_computed (values[i]))
-        {
-          result->is_computed = FALSE;
-          break;
-        }
+        result->is_computed = FALSE;
+
+      if (gtk_css_value_contains_variables (values[i]))
+        result->contains_variables = TRUE;
+
+      if (gtk_css_value_contains_current_color (values[i]))
+        result->contains_current_color = TRUE;
+
+      if (!result->is_computed && result->contains_variables && result->contains_current_color)
+        break;
     }
 
   return result;
@@ -426,7 +475,7 @@ _gtk_css_array_value_parse (GtkCssParser *parser,
     if (value == NULL)
       {
         for (i = 0; i < n_values; i ++)
-          _gtk_css_value_unref (values[i]);
+          gtk_css_value_unref (values[i]);
 
         return NULL;
       }
@@ -466,4 +515,3 @@ _gtk_css_array_value_get_n_values (const GtkCssValue *value)
 
   return value->n_values;
 }
-
