@@ -17,8 +17,7 @@
  * Authors: Alexander Larsson <alexl@gnome.org>
  */
 
-#ifndef __GTK_CSS_DIMENSION_VALUE_PRIVATE_H__
-#define __GTK_CSS_DIMENSION_VALUE_PRIVATE_H__
+#pragma once
 
 #include "gtkcssnumbervalueprivate.h"
 
@@ -28,4 +27,3 @@ GtkCssValue *   gtk_css_dimension_value_parse       (GtkCssParser           *par
                                                      GtkCssNumberParseFlags  flags);
 G_END_DECLS
 
-#endif /* __GTK_CSS_DIMENSION_VALUE_PRIVATE_H__ */
