@@ -65,34 +65,34 @@ gtk_css_filter_clear (GtkCssFilter *filter)
   switch (filter->type)
     {
     case GTK_CSS_FILTER_BRIGHTNESS:
-      _gtk_css_value_unref (filter->brightness.value);
+      gtk_css_value_unref (filter->brightness.value);
       break;
     case GTK_CSS_FILTER_CONTRAST:
-      _gtk_css_value_unref (filter->contrast.value);
+      gtk_css_value_unref (filter->contrast.value);
       break;
     case GTK_CSS_FILTER_GRAYSCALE:
-      _gtk_css_value_unref (filter->grayscale.value);
+      gtk_css_value_unref (filter->grayscale.value);
       break;
     case GTK_CSS_FILTER_HUE_ROTATE:
-      _gtk_css_value_unref (filter->hue_rotate.value);
+      gtk_css_value_unref (filter->hue_rotate.value);
       break;
     case GTK_CSS_FILTER_INVERT:
-      _gtk_css_value_unref (filter->invert.value);
+      gtk_css_value_unref (filter->invert.value);
       break;
     case GTK_CSS_FILTER_OPACITY:
-      _gtk_css_value_unref (filter->opacity.value);
+      gtk_css_value_unref (filter->opacity.value);
       break;
     case GTK_CSS_FILTER_SATURATE:
-      _gtk_css_value_unref (filter->saturate.value);
+      gtk_css_value_unref (filter->saturate.value);
       break;
     case GTK_CSS_FILTER_SEPIA:
-      _gtk_css_value_unref (filter->sepia.value);
+      gtk_css_value_unref (filter->sepia.value);
       break;
     case GTK_CSS_FILTER_BLUR:
-      _gtk_css_value_unref (filter->blur.value);
+      gtk_css_value_unref (filter->blur.value);
       break;
     case GTK_CSS_FILTER_DROP_SHADOW:
-      _gtk_css_value_unref (filter->drop_shadow.value);
+      gtk_css_value_unref (filter->drop_shadow.value);
       break;
     case GTK_CSS_FILTER_NONE:
     default:
@@ -108,31 +108,31 @@ gtk_css_filter_init_identity (GtkCssFilter       *filter,
   switch (other->type)
     {
     case GTK_CSS_FILTER_BRIGHTNESS:
-      filter->brightness.value = _gtk_css_number_value_new (1, GTK_CSS_NUMBER);
+      filter->brightness.value = gtk_css_number_value_new (1, GTK_CSS_NUMBER);
       break;
     case GTK_CSS_FILTER_CONTRAST:
-      filter->contrast.value = _gtk_css_number_value_new (1, GTK_CSS_NUMBER);
+      filter->contrast.value = gtk_css_number_value_new (1, GTK_CSS_NUMBER);
       break;
     case GTK_CSS_FILTER_GRAYSCALE:
-      filter->grayscale.value = _gtk_css_number_value_new (0, GTK_CSS_NUMBER);
+      filter->grayscale.value = gtk_css_number_value_new (0, GTK_CSS_NUMBER);
       break;
     case GTK_CSS_FILTER_HUE_ROTATE:
-      filter->hue_rotate.value = _gtk_css_number_value_new (0, GTK_CSS_DEG);
+      filter->hue_rotate.value = gtk_css_number_value_new (0, GTK_CSS_DEG);
       break;
     case GTK_CSS_FILTER_INVERT:
-      filter->invert.value = _gtk_css_number_value_new (0, GTK_CSS_NUMBER);
+      filter->invert.value = gtk_css_number_value_new (0, GTK_CSS_NUMBER);
       break;
     case GTK_CSS_FILTER_OPACITY:
-      filter->opacity.value = _gtk_css_number_value_new (1, GTK_CSS_NUMBER);
+      filter->opacity.value = gtk_css_number_value_new (1, GTK_CSS_NUMBER);
       break;
     case GTK_CSS_FILTER_SATURATE:
-      filter->saturate.value = _gtk_css_number_value_new (1, GTK_CSS_NUMBER);
+      filter->saturate.value = gtk_css_number_value_new (1, GTK_CSS_NUMBER);
       break;
     case GTK_CSS_FILTER_SEPIA:
-      filter->sepia.value = _gtk_css_number_value_new (0, GTK_CSS_NUMBER);
+      filter->sepia.value = gtk_css_number_value_new (0, GTK_CSS_NUMBER);
       break;
     case GTK_CSS_FILTER_BLUR:
-      filter->blur.value = _gtk_css_number_value_new (0, GTK_CSS_PX);
+      filter->blur.value = gtk_css_number_value_new (0, GTK_CSS_PX);
       break;
     case GTK_CSS_FILTER_DROP_SHADOW:
       filter->drop_shadow.value = gtk_css_shadow_value_new_filter (other->drop_shadow.value);
@@ -160,19 +160,19 @@ gtk_css_filter_get_matrix (const GtkCssFilter *filter,
   switch (filter->type)
     {
     case GTK_CSS_FILTER_BRIGHTNESS:
-      value = _gtk_css_number_value_get (filter->brightness.value, 1.0);
+      value = gtk_css_number_value_get (filter->brightness.value, 1.0);
       graphene_matrix_init_scale (matrix, value, value, value);
       graphene_vec4_init (offset, 0.0, 0.0, 0.0, 0.0);
       break;
 
     case GTK_CSS_FILTER_CONTRAST:
-      value = _gtk_css_number_value_get (filter->contrast.value, 1.0);
+      value = gtk_css_number_value_get (filter->contrast.value, 1.0);
       graphene_matrix_init_scale (matrix, value, value, value);
       graphene_vec4_init (offset, 0.5 - 0.5 * value, 0.5 - 0.5 * value, 0.5 - 0.5 * value, 0.0);
       break;
 
     case GTK_CSS_FILTER_GRAYSCALE:
-      value = _gtk_css_number_value_get (filter->grayscale.value, 1.0);
+      value = gtk_css_number_value_get (filter->grayscale.value, 1.0);
       graphene_matrix_init_from_float (matrix, (float[16]) {
                                            1.0 - (1.0 - R) * value, R * value, R * value, 0.0,
                                            G * value, 1.0 - (1.0 - G) * value, G * value, 0.0,
@@ -185,7 +185,7 @@ gtk_css_filter_get_matrix (const GtkCssFilter *filter,
     case GTK_CSS_FILTER_HUE_ROTATE:
       {
         double c, s;
-        value = _gtk_css_number_value_get (filter->grayscale.value, 1.0) * G_PI / 180.0;
+        value = gtk_css_number_value_get (filter->grayscale.value, 1.0) * G_PI / 180.0;
         c = cos (value);
         s = sin (value);
         graphene_matrix_init_from_float (matrix, (float[16]) {
@@ -208,13 +208,13 @@ gtk_css_filter_get_matrix (const GtkCssFilter *filter,
       break;
 
     case GTK_CSS_FILTER_INVERT:
-      value = _gtk_css_number_value_get (filter->invert.value, 1.0);
+      value = gtk_css_number_value_get (filter->invert.value, 1.0);
       graphene_matrix_init_scale (matrix, 1.0 - 2 * value, 1.0 - 2 * value, 1.0 - 2 * value);
       graphene_vec4_init (offset, value, value, value, 0.0);
       break;
 
     case GTK_CSS_FILTER_OPACITY:
-      value = _gtk_css_number_value_get (filter->invert.value, 1.0);
+      value = gtk_css_number_value_get (filter->opacity.value, 1.0);
       graphene_matrix_init_from_float (matrix, (float[16]) {
                                            1.0, 0.0, 0.0, 0.0,
                                            0.0, 1.0, 0.0, 0.0,
@@ -225,7 +225,7 @@ gtk_css_filter_get_matrix (const GtkCssFilter *filter,
       break;
 
     case GTK_CSS_FILTER_SATURATE:
-      value = _gtk_css_number_value_get (filter->saturate.value, 1.0);
+      value = gtk_css_number_value_get (filter->saturate.value, 1.0);
       graphene_matrix_init_from_float (matrix, (float[16]) {
                                            R + (1.0 - R) * value, R - R * value, R - R * value, 0.0,
                                            G - G * value, G + (1.0 - G) * value, G - G * value, 0.0,
@@ -236,7 +236,7 @@ gtk_css_filter_get_matrix (const GtkCssFilter *filter,
       break;
 
     case GTK_CSS_FILTER_SEPIA:
-      value = _gtk_css_number_value_get (filter->sepia.value, 1.0);
+      value = gtk_css_number_value_get (filter->sepia.value, 1.0);
       graphene_matrix_init_from_float (matrix, (float[16]) {
                                            1.0 - 0.607 * value, 0.349 * value, 0.272 * value, 0.0,
                                            0.769 * value, 1.0 - 0.314 * value, 0.534 * value, 0.0,
@@ -307,55 +307,53 @@ gtk_css_value_filter_free (GtkCssValue *value)
 
 /* returns TRUE if dest == src */
 static gboolean
-gtk_css_filter_compute (GtkCssFilter     *dest,
-                        GtkCssFilter     *src,
-                        guint             property_id,
-                        GtkStyleProvider *provider,
-                        GtkCssStyle      *style,
-                        GtkCssStyle      *parent_style)
+gtk_css_filter_compute (GtkCssFilter         *dest,
+                        GtkCssFilter         *src,
+                        guint                 property_id,
+                        GtkCssComputeContext *context)
 {
   dest->type = src->type;
 
   switch (src->type)
     {
     case GTK_CSS_FILTER_BRIGHTNESS:
-      dest->brightness.value = _gtk_css_value_compute (src->brightness.value, property_id, provider, style, parent_style);
+      dest->brightness.value = gtk_css_value_compute (src->brightness.value, property_id, context);
       return dest->brightness.value == src->brightness.value;
 
     case GTK_CSS_FILTER_CONTRAST:
-      dest->contrast.value = _gtk_css_value_compute (src->contrast.value, property_id, provider, style, parent_style);
+      dest->contrast.value = gtk_css_value_compute (src->contrast.value, property_id, context);
       return dest->contrast.value == src->contrast.value;
 
     case GTK_CSS_FILTER_GRAYSCALE:
-      dest->grayscale.value = _gtk_css_value_compute (src->grayscale.value, property_id, provider, style, parent_style);
+      dest->grayscale.value = gtk_css_value_compute (src->grayscale.value, property_id, context);
       return dest->grayscale.value == src->grayscale.value;
 
     case GTK_CSS_FILTER_HUE_ROTATE:
-      dest->hue_rotate.value = _gtk_css_value_compute (src->hue_rotate.value, property_id, provider, style, parent_style);
+      dest->hue_rotate.value = gtk_css_value_compute (src->hue_rotate.value, property_id, context);
       return dest->hue_rotate.value == src->hue_rotate.value;
 
     case GTK_CSS_FILTER_INVERT:
-      dest->invert.value = _gtk_css_value_compute (src->invert.value, property_id, provider, style, parent_style);
+      dest->invert.value = gtk_css_value_compute (src->invert.value, property_id, context);
       return dest->invert.value == src->invert.value;
 
     case GTK_CSS_FILTER_OPACITY:
-      dest->opacity.value = _gtk_css_value_compute (src->opacity.value, property_id, provider, style, parent_style);
+      dest->opacity.value = gtk_css_value_compute (src->opacity.value, property_id, context);
       return dest->opacity.value == src->opacity.value;
 
     case GTK_CSS_FILTER_SATURATE:
-      dest->saturate.value = _gtk_css_value_compute (src->saturate.value, property_id, provider, style, parent_style);
+      dest->saturate.value = gtk_css_value_compute (src->saturate.value, property_id, context);
       return dest->saturate.value == src->saturate.value;
 
     case GTK_CSS_FILTER_SEPIA:
-      dest->sepia.value = _gtk_css_value_compute (src->sepia.value, property_id, provider, style, parent_style);
+      dest->sepia.value = gtk_css_value_compute (src->sepia.value, property_id, context);
       return dest->sepia.value == src->sepia.value;
 
     case GTK_CSS_FILTER_BLUR:
-      dest->blur.value = _gtk_css_value_compute (src->blur.value, property_id, provider, style, parent_style);
+      dest->blur.value = gtk_css_value_compute (src->blur.value, property_id, context);
       return dest->blur.value == src->blur.value;
 
     case GTK_CSS_FILTER_DROP_SHADOW:
-      dest->drop_shadow.value = _gtk_css_value_compute (src->drop_shadow.value, property_id, provider, style, parent_style);
+      dest->drop_shadow.value = gtk_css_value_compute (src->drop_shadow.value, property_id, context);
       return dest->drop_shadow.value == src->drop_shadow.value;
 
     case GTK_CSS_FILTER_NONE:
@@ -366,11 +364,9 @@ gtk_css_filter_compute (GtkCssFilter     *dest,
 }
 
 static GtkCssValue *
-gtk_css_value_filter_compute (GtkCssValue      *value,
-                              guint             property_id,
-                              GtkStyleProvider *provider,
-                              GtkCssStyle      *style,
-                              GtkCssStyle      *parent_style)
+gtk_css_value_filter_compute (GtkCssValue          *value,
+                              guint                 property_id,
+                              GtkCssComputeContext *context)
 {
   GtkCssValue *result;
   gboolean changes;
@@ -378,7 +374,7 @@ gtk_css_value_filter_compute (GtkCssValue      *value,
 
   /* Special case the 99% case of "none" */
   if (gtk_css_filter_value_is_none (value))
-    return _gtk_css_value_ref (value);
+    return gtk_css_value_ref (value);
 
   changes = FALSE;
   result = gtk_css_filter_value_alloc (value->n_filters);
@@ -388,15 +384,13 @@ gtk_css_value_filter_compute (GtkCssValue      *value,
       changes |= !gtk_css_filter_compute (&result->filters[i],
                                           &value->filters[i],
                                           property_id,
-                                          provider,
-                                          style,
-                                          parent_style);
+                                          context);
     }
 
   if (!changes)
     {
-      _gtk_css_value_unref (result);
-      result = _gtk_css_value_ref (value);
+      gtk_css_value_unref (result);
+      result = gtk_css_value_ref (value);
     }
 
   return result;
@@ -412,34 +406,34 @@ gtk_css_filter_equal (const GtkCssFilter *filter1,
   switch (filter1->type)
     {
     case GTK_CSS_FILTER_BRIGHTNESS:
-      return _gtk_css_value_equal (filter1->brightness.value, filter2->brightness.value);
+      return gtk_css_value_equal (filter1->brightness.value, filter2->brightness.value);
 
     case GTK_CSS_FILTER_CONTRAST:
-      return _gtk_css_value_equal (filter1->contrast.value, filter2->contrast.value);
+      return gtk_css_value_equal (filter1->contrast.value, filter2->contrast.value);
 
     case GTK_CSS_FILTER_GRAYSCALE:
-      return _gtk_css_value_equal (filter1->grayscale.value, filter2->grayscale.value);
+      return gtk_css_value_equal (filter1->grayscale.value, filter2->grayscale.value);
 
     case GTK_CSS_FILTER_HUE_ROTATE:
-      return _gtk_css_value_equal (filter1->hue_rotate.value, filter2->hue_rotate.value);
+      return gtk_css_value_equal (filter1->hue_rotate.value, filter2->hue_rotate.value);
 
     case GTK_CSS_FILTER_INVERT:
-      return _gtk_css_value_equal (filter1->invert.value, filter2->invert.value);
+      return gtk_css_value_equal (filter1->invert.value, filter2->invert.value);
 
     case GTK_CSS_FILTER_OPACITY:
-      return _gtk_css_value_equal (filter1->opacity.value, filter2->opacity.value);
+      return gtk_css_value_equal (filter1->opacity.value, filter2->opacity.value);
 
     case GTK_CSS_FILTER_SATURATE:
-      return _gtk_css_value_equal (filter1->saturate.value, filter2->saturate.value);
+      return gtk_css_value_equal (filter1->saturate.value, filter2->saturate.value);
 
     case GTK_CSS_FILTER_SEPIA:
-      return _gtk_css_value_equal (filter1->sepia.value, filter2->sepia.value);
+      return gtk_css_value_equal (filter1->sepia.value, filter2->sepia.value);
 
     case GTK_CSS_FILTER_BLUR:
-      return _gtk_css_value_equal (filter1->blur.value, filter2->blur.value);
+      return gtk_css_value_equal (filter1->blur.value, filter2->blur.value);
 
     case GTK_CSS_FILTER_DROP_SHADOW:
-      return _gtk_css_value_equal (filter1->drop_shadow.value, filter2->drop_shadow.value);
+      return gtk_css_value_equal (filter1->drop_shadow.value, filter2->drop_shadow.value);
 
     case GTK_CSS_FILTER_NONE:
     default:
@@ -494,43 +488,43 @@ gtk_css_filter_transition (GtkCssFilter       *result,
   switch (start->type)
     {
     case GTK_CSS_FILTER_BRIGHTNESS:
-      result->brightness.value = _gtk_css_value_transition (start->brightness.value, end->brightness.value, property_id, progress);
+      result->brightness.value = gtk_css_value_transition (start->brightness.value, end->brightness.value, property_id, progress);
       break;
 
     case GTK_CSS_FILTER_CONTRAST:
-      result->contrast.value = _gtk_css_value_transition (start->contrast.value, end->contrast.value, property_id, progress);
+      result->contrast.value = gtk_css_value_transition (start->contrast.value, end->contrast.value, property_id, progress);
       break;
 
     case GTK_CSS_FILTER_GRAYSCALE:
-      result->grayscale.value = _gtk_css_value_transition (start->grayscale.value, end->grayscale.value, property_id, progress);
+      result->grayscale.value = gtk_css_value_transition (start->grayscale.value, end->grayscale.value, property_id, progress);
       break;
 
     case GTK_CSS_FILTER_HUE_ROTATE:
-      result->hue_rotate.value = _gtk_css_value_transition (start->hue_rotate.value, end->hue_rotate.value, property_id, progress);
+      result->hue_rotate.value = gtk_css_value_transition (start->hue_rotate.value, end->hue_rotate.value, property_id, progress);
       break;
 
     case GTK_CSS_FILTER_INVERT:
-      result->invert.value = _gtk_css_value_transition (start->invert.value, end->invert.value, property_id, progress);
+      result->invert.value = gtk_css_value_transition (start->invert.value, end->invert.value, property_id, progress);
       break;
 
     case GTK_CSS_FILTER_OPACITY:
-      result->opacity.value = _gtk_css_value_transition (start->opacity.value, end->opacity.value, property_id, progress);
+      result->opacity.value = gtk_css_value_transition (start->opacity.value, end->opacity.value, property_id, progress);
       break;
 
     case GTK_CSS_FILTER_SATURATE:
-      result->saturate.value = _gtk_css_value_transition (start->saturate.value, end->saturate.value, property_id, progress);
+      result->saturate.value = gtk_css_value_transition (start->saturate.value, end->saturate.value, property_id, progress);
       break;
 
     case GTK_CSS_FILTER_SEPIA:
-      result->sepia.value = _gtk_css_value_transition (start->sepia.value, end->sepia.value, property_id, progress);
+      result->sepia.value = gtk_css_value_transition (start->sepia.value, end->sepia.value, property_id, progress);
       break;
 
     case GTK_CSS_FILTER_BLUR:
-      result->blur.value = _gtk_css_value_transition (start->blur.value, end->blur.value, property_id, progress);
+      result->blur.value = gtk_css_value_transition (start->blur.value, end->blur.value, property_id, progress);
       break;
 
     case GTK_CSS_FILTER_DROP_SHADOW:
-      result->drop_shadow.value = _gtk_css_value_transition (start->drop_shadow.value, end->drop_shadow.value, property_id, progress);
+      result->drop_shadow.value = gtk_css_value_transition (start->drop_shadow.value, end->drop_shadow.value, property_id, progress);
       break;
 
     case GTK_CSS_FILTER_NONE:
@@ -552,7 +546,7 @@ gtk_css_value_filter_transition (GtkCssValue *start,
   if (gtk_css_filter_value_is_none (start))
     {
       if (gtk_css_filter_value_is_none (end))
-        return _gtk_css_value_ref (start);
+        return gtk_css_value_ref (start);
 
       n = 0;
     }
@@ -626,61 +620,61 @@ gtk_css_filter_print (const GtkCssFilter *filter,
     {
     case GTK_CSS_FILTER_BRIGHTNESS:
       g_string_append (string, "brightness(");
-      _gtk_css_value_print (filter->brightness.value, string);
+      gtk_css_value_print (filter->brightness.value, string);
       g_string_append (string, ")");
       break;
 
     case GTK_CSS_FILTER_CONTRAST:
       g_string_append (string, "contrast(");
-      _gtk_css_value_print (filter->contrast.value, string);
+      gtk_css_value_print (filter->contrast.value, string);
       g_string_append (string, ")");
       break;
 
     case GTK_CSS_FILTER_GRAYSCALE:
       g_string_append (string, "grayscale(");
-      _gtk_css_value_print (filter->grayscale.value, string);
+      gtk_css_value_print (filter->grayscale.value, string);
       g_string_append (string, ")");
       break;
 
     case GTK_CSS_FILTER_HUE_ROTATE:
       g_string_append (string, "hue-rotate(");
-      _gtk_css_value_print (filter->hue_rotate.value, string);
+      gtk_css_value_print (filter->hue_rotate.value, string);
       g_string_append (string, ")");
       break;
 
     case GTK_CSS_FILTER_INVERT:
       g_string_append (string, "invert(");
-      _gtk_css_value_print (filter->invert.value, string);
+      gtk_css_value_print (filter->invert.value, string);
       g_string_append (string, ")");
       break;
 
     case GTK_CSS_FILTER_OPACITY:
       g_string_append (string, "opacity(");
-      _gtk_css_value_print (filter->opacity.value, string);
+      gtk_css_value_print (filter->opacity.value, string);
       g_string_append (string, ")");
       break;
 
     case GTK_CSS_FILTER_SATURATE:
       g_string_append (string, "saturate(");
-      _gtk_css_value_print (filter->saturate.value, string);
+      gtk_css_value_print (filter->saturate.value, string);
       g_string_append (string, ")");
       break;
 
     case GTK_CSS_FILTER_SEPIA:
       g_string_append (string, "sepia(");
-      _gtk_css_value_print (filter->sepia.value, string);
+      gtk_css_value_print (filter->sepia.value, string);
       g_string_append (string, ")");
       break;
 
     case GTK_CSS_FILTER_BLUR:
       g_string_append (string, "blur(");
-      _gtk_css_value_print (filter->blur.value, string);
+      gtk_css_value_print (filter->blur.value, string);
       g_string_append (string, ")");
       break;
 
     case GTK_CSS_FILTER_DROP_SHADOW:
       g_string_append (string, "drop-shadow(");
-      _gtk_css_value_print (filter->drop_shadow.value, string);
+      gtk_css_value_print (filter->drop_shadow.value, string);
       g_string_append (string, ")");
       break;
 
@@ -716,6 +710,7 @@ static const GtkCssValueClass GTK_CSS_VALUE_FILTER = {
   "GtkCssFilterValue",
   gtk_css_value_filter_free,
   gtk_css_value_filter_compute,
+  NULL,
   gtk_css_value_filter_equal,
   gtk_css_value_filter_transition,
   NULL,
@@ -723,7 +718,7 @@ static const GtkCssValueClass GTK_CSS_VALUE_FILTER = {
   gtk_css_value_filter_print
 };
 
-static GtkCssValue filter_none_singleton = { &GTK_CSS_VALUE_FILTER, 1, TRUE, 0, {  { GTK_CSS_FILTER_NONE } } };
+static GtkCssValue filter_none_singleton = { &GTK_CSS_VALUE_FILTER, 1, 1, 0, 0, 0, {  { GTK_CSS_FILTER_NONE } } };
 
 static GtkCssValue *
 gtk_css_filter_value_alloc (guint n_filters)
@@ -732,7 +727,7 @@ gtk_css_filter_value_alloc (guint n_filters)
 
   g_return_val_if_fail (n_filters > 0, NULL);
 
-  result = _gtk_css_value_alloc (&GTK_CSS_VALUE_FILTER, sizeof (GtkCssValue) + sizeof (GtkCssFilter) * (n_filters - 1));
+  result = gtk_css_value_alloc (&GTK_CSS_VALUE_FILTER, sizeof (GtkCssValue) + sizeof (GtkCssFilter) * (n_filters - 1));
   result->n_filters = n_filters;
 
   return result;
@@ -741,7 +736,7 @@ gtk_css_filter_value_alloc (guint n_filters)
 GtkCssValue *
 gtk_css_filter_value_new_none (void)
 {
-  return _gtk_css_value_ref (&filter_none_singleton);
+  return gtk_css_value_ref (&filter_none_singleton);
 }
 
 static gboolean
@@ -757,7 +752,7 @@ gtk_css_filter_parse_number (GtkCssParser *parser,
 {
   GtkCssValue **values = data;
 
-  values[n] = _gtk_css_number_value_parse (parser, GTK_CSS_PARSE_NUMBER | GTK_CSS_PARSE_PERCENT | GTK_CSS_POSITIVE_ONLY);
+  values[n] = gtk_css_number_value_parse (parser, GTK_CSS_PARSE_NUMBER | GTK_CSS_PARSE_PERCENT | GTK_CSS_POSITIVE_ONLY);
   if (values[n] == NULL)
     return 0;
 
@@ -771,7 +766,7 @@ gtk_css_filter_parse_length (GtkCssParser *parser,
 {
   GtkCssValue **values = data;
 
-  values[n] = _gtk_css_number_value_parse (parser, GTK_CSS_PARSE_LENGTH | GTK_CSS_POSITIVE_ONLY);
+  values[n] = gtk_css_number_value_parse (parser, GTK_CSS_PARSE_LENGTH | GTK_CSS_POSITIVE_ONLY);
   if (values[n] == NULL)
     return 0;
 
@@ -785,7 +780,7 @@ gtk_css_filter_parse_angle (GtkCssParser *parser,
 {
   GtkCssValue **values = data;
 
-  values[n] = _gtk_css_number_value_parse (parser, GTK_CSS_PARSE_ANGLE);
+  values[n] = gtk_css_number_value_parse (parser, GTK_CSS_PARSE_ANGLE);
   if (values[n] == NULL)
     return 0;
 
@@ -963,7 +958,7 @@ gtk_css_filter_value_push_snapshot (const GtkCssValue *filter,
         {
           if (filter->filters[j].type == GTK_CSS_FILTER_BLUR)
             {
-              double std_dev = _gtk_css_number_value_get (filter->filters[j].blur.value, 100.0);
+              double std_dev = gtk_css_number_value_get (filter->filters[j].blur.value, 100.0);
               gtk_snapshot_push_blur (snapshot, 2 * std_dev);
             }
           else if (filter->filters[j].type == GTK_CSS_FILTER_DROP_SHADOW)
