@@ -76,6 +76,11 @@
 #include "gtkfilter.h"
 #include "gtkprivate.h"
 
+#ifdef GDK_WINDOWING_ANDROID
+#include "android/gdkandroidinit-private.h"
+#include "android/gdkandroidutils-private.h"
+#endif
+
 #include <glib/gi18n-lib.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <string.h>
@@ -442,6 +447,8 @@ gtk_file_filter_buildable_custom_tag_start (GtkBuildable       *buildable,
 
   if (strcmp (tagname, "mime-types") == 0)
     {
+      gtk_buildable_tag_deprecation_warning (buildable,builder, "mime-types", "mime-types");
+
       data = g_new0 (SubParserData, 1);
       data->string = g_string_new ("");
       data->type = PARSE_MIME_TYPES;
@@ -453,6 +460,8 @@ gtk_file_filter_buildable_custom_tag_start (GtkBuildable       *buildable,
     }
   else if (strcmp (tagname, "patterns") == 0)
     {
+      gtk_buildable_tag_deprecation_warning (buildable,builder, "patterns", "patterns");
+
       data = g_new0 (SubParserData, 1);
       data->string = g_string_new ("");
       data->type = PARSE_PATTERNS;
@@ -464,6 +473,8 @@ gtk_file_filter_buildable_custom_tag_start (GtkBuildable       *buildable,
     }
   else if (strcmp (tagname, "suffixes") == 0)
     {
+      gtk_buildable_tag_deprecation_warning (buildable,builder, "suffixes", "suffixes");
+
       data = g_new0 (SubParserData, 1);
       data->string = g_string_new ("");
       data->type = PARSE_SUFFIXES;
@@ -664,7 +675,14 @@ gtk_file_filter_add_pattern (GtkFileFilter *filter,
  *
  * Adds a suffix match rule to a filter.
  *
- * This is similar to adding a match for the pattern "*.@suffix".
+ * This is similar to adding a match for the pattern "*.@suffix"
+ *
+ * An exaple to filter files with the suffix ".sub":
+ * ```c
+ * gtk_file_filter_add_suffix (filter, "sub");
+ * ```
+ *
+ * Filters with multiple dots are allowed.
  *
  * In contrast to pattern matches, suffix matches
  * are *always* case-insensitive.
@@ -809,6 +827,48 @@ NSArray * _gtk_file_filter_get_as_pattern_nsstrings (GtkFileFilter *filter)
 }
 #endif
 
+#ifdef GDK_WINDOWING_ANDROID
+void _gtk_file_filter_store_types_in_list (GtkFileFilter *filter, jobject list)
+{
+  JNIEnv *env = gdk_android_get_env ();
+  (*env)->PushLocalFrame (env, 2);
+
+  for (GSList *i = filter->rules; i != NULL; i = i->next)
+    {
+      FilterRule *rule = i->data;
+
+      switch (rule->type)
+        {
+        case FILTER_RULE_MIME_TYPE:
+        case FILTER_RULE_PIXBUF_FORMATS:
+          {
+            for (gsize j = 0; rule->u.content_types[j] != NULL; j++)
+              {
+                jstring content_type = gdk_android_utf8_to_java (rule->u.content_types[j]);
+                jstring normalized = (*env)->CallStaticObjectMethod (env, gdk_android_get_java_cache ()->a_intent.klass,
+                                                                     gdk_android_get_java_cache ()->a_intent.normalize_mimetype,
+                                                                     content_type);
+                (*env)->CallBooleanMethod (env, list,
+                                           gdk_android_get_java_cache ()->j_list.add,
+                                           normalized);
+                (*env)->DeleteLocalRef (env, normalized);
+                (*env)->DeleteLocalRef (env, content_type);
+              }
+          }
+          break;
+
+        case FILTER_RULE_SUFFIX:
+        case FILTER_RULE_PATTERN:
+        default:
+          // unsupported
+          break;
+       }
+    }
+
+  (*env)->PopLocalFrame (env, NULL);
+}
+#endif
+
 char **
 _gtk_file_filter_get_as_patterns (GtkFileFilter *filter)
 {
