@@ -31,7 +31,8 @@ typedef enum {
 
 typedef struct _GtkKineticScrolling GtkKineticScrolling;
 
-GtkKineticScrolling *    gtk_kinetic_scrolling_new  (double                lower,
+GtkKineticScrolling *    gtk_kinetic_scrolling_new  (gint64                frame_time,
+                                                     double                lower,
                                                      double                upper,
                                                      double                overshoot_width,
                                                      double                decel_friction,
@@ -45,7 +46,7 @@ GtkKineticScrollingChange gtk_kinetic_scrolling_update_size (GtkKineticScrolling
                                                              double               upper);
 
 gboolean                 gtk_kinetic_scrolling_tick (GtkKineticScrolling  *data,
-                                                     double                time_delta,
+                                                     gint64                frame_time,
                                                      double               *position,
                                                      double               *velocity);
 
