@@ -32,7 +32,6 @@
 #include <gio/gio.h>
 #include "x11/gdkx.h"
 #include <X11/Xatom.h>
-#include <gtk/gtkicontheme.h>
 #include <glib/gi18n-lib.h>
 
 /* for the kill(2) system call and errno - POSIX.1-2001 and later */
@@ -46,6 +45,8 @@
 
 #include "gtkmountoperationprivate.h"
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 /* ---------------------------------------------------------------------------------------------------- */
 /* these functions are based on code from libwnck (LGPLv2) */
 
