@@ -21,10 +21,12 @@
 /**
  * GtkPopoverMenuBar:
  *
- * `GtkPopoverMenuBar` presents a horizontal bar of items that pop
- * up popover menus when clicked.
+ * Presents a horizontal bar of items that pop up menus when clicked.
  *
- * ![An example GtkPopoverMenuBar](menubar.png)
+ * <picture>
+ *   <source srcset="menubar-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkPopoverMenuBar" src="menubar.png">
+ * </picture>
  *
  * The only way to create instances of `GtkPopoverMenuBar` is
  * from a `GMenuModel`.
@@ -47,9 +49,9 @@
  *
  * # Accessibility
  *
- * `GtkPopoverMenuBar` uses the %GTK_ACCESSIBLE_ROLE_MENU_BAR role,
- * the menu items use the %GTK_ACCESSIBLE_ROLE_MENU_ITEM role and
- * the menus use the %GTK_ACCESSIBLE_ROLE_MENU role.
+ * `GtkPopoverMenuBar` uses the [enum@Gtk.AccessibleRole.menu_bar] role,
+ * the menu items use the [enum@Gtk.AccessibleRole.menu_item] role and
+ * the menus use the [enum@Gtk.AccessibleRole.menu] role.
  */
 
 
@@ -160,9 +162,11 @@ set_active_item (GtkPopoverMenuBar     *bar,
 
   if (bar->active_item)
     {
+      GtkStateFlags state = gtk_widget_get_state_flags (GTK_WIDGET (bar));
+
       if (popup || (was_popup && changed))
         open_submenu (bar->active_item);
-      else if (changed)
+      else if (changed && (state & GTK_STATE_FLAG_FOCUS_WITHIN))
         gtk_widget_grab_focus (GTK_WIDGET (bar->active_item));
     }
 }
@@ -624,7 +628,7 @@ gtk_popover_menu_bar_class_init (GtkPopoverMenuBarClass *klass)
   widget_class->focus = gtk_popover_menu_bar_focus;
 
   /**
-   * GtkPopoverMenuBar:menu-model: (attributes org.gtk.Property.get=gtk_popover_menu_bar_get_menu_model org.gtk.Property.set=gtk_popover_menu_bar_set_menu_model)
+   * GtkPopoverMenuBar:menu-model:
    *
    * The `GMenuModel` from which the menu bar is created.
    *
@@ -695,7 +699,7 @@ gtk_popover_menu_bar_new_from_model (GMenuModel *model)
 }
 
 /**
- * gtk_popover_menu_bar_set_menu_model: (attributes org.gtk.Method.set_property=menu-model)
+ * gtk_popover_menu_bar_set_menu_model:
  * @bar: a `GtkPopoverMenuBar`
  * @model: (nullable): a `GMenuModel`
  *
@@ -738,7 +742,7 @@ gtk_popover_menu_bar_set_menu_model (GtkPopoverMenuBar *bar,
 }
 
 /**
- * gtk_popover_menu_bar_get_menu_model: (attributes org.gtk.Method.get_property=menu-model)
+ * gtk_popover_menu_bar_get_menu_model:
  * @bar: a `GtkPopoverMenuBar`
  *
  * Returns the model from which the contents of @bar are taken.
