@@ -71,6 +71,7 @@
 #include <xkbcommon/xkbcommon.h>
 #include "wayland/gdkdisplay-wayland.h"
 #include "wayland/gdkwaylandcolor-private.h"
+#include "gtk/gtkimcontextwayland.h"
 #endif
 
 #ifdef GDK_WINDOWING_BROADWAY
@@ -1170,12 +1171,15 @@ static const char *env_list[] = {
   "LANGUAGE",
   "LC_ALL",
   "LC_CTYPE",
+  "LIBGL_ALWAYS_SOFTWARE",
   "LPDEST",
+  "MESA_VK_DEVICE_SELECT",
   "PANGOCAIRO_BACKEND",
   "PANGO_LANGUAGE",
   "PRINTER",
   "SECMEM_FORCE_FALLBACK",
   "WAYLAND_DISPLAY",
+  "XDG_ACTIVATION_TOKEN",
   "XDG_DATA_HOME",
   "XDG_DATA_DIRS",
   "XDG_RUNTIME_DIR",
@@ -1308,6 +1312,7 @@ add_wayland_protocols (GdkDisplay          *display,
       append_wayland_protocol_row (gen, d->color ? gdk_wayland_color_get_color_manager (d->color) : NULL);
       append_wayland_protocol_row (gen, (struct wl_proxy *)d->system_bell);
       append_wayland_protocol_row (gen, (struct wl_proxy *)d->cursor_shape);
+      append_wayland_protocol_row (gen, gtk_im_context_wayland_get_text_protocol (display));
     }
 }
 
@@ -1348,6 +1353,8 @@ dump_wayland_protocols (GdkDisplay *display,
       append_wayland_protocol (string, (struct wl_proxy *)d->single_pixel_buffer, &count);
       append_wayland_protocol (string, d->color ? gdk_wayland_color_get_color_manager (d->color) : NULL, &count);
       append_wayland_protocol (string, (struct wl_proxy *)d->system_bell, &count);
+      append_wayland_protocol (string, (struct wl_proxy *)d->cursor_shape, &count);
+      append_wayland_protocol (string , gtk_im_context_wayland_get_text_protocol (display), &count);
 
       g_string_append (string, " |\n");
     }
