@@ -1,6 +1,8 @@
 #include <string.h>
 #include <gtk/gtk.h>
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 static const char *css =
 ".overlay-green {"
 "  background-image: none;"
@@ -538,28 +540,28 @@ main (int argc, char *argv[])
                                               GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 
   win1 = test_nonzerox ();
-  gtk_widget_show (win1);
+  gtk_window_present (GTK_WINDOW (win1));
 
   win2 = test_relative ();
-  gtk_widget_show (win2);
+  gtk_window_present (GTK_WINDOW (win2));
 
   win3 = test_fullwidth ();
-  gtk_widget_show (win3);
+  gtk_window_present (GTK_WINDOW (win3));
 
   win4 = test_scrolling ();
-  gtk_widget_show (win4);
+  gtk_window_present (GTK_WINDOW (win4));
 
   win5 = test_builder ();
-  gtk_widget_show (win5);
+  gtk_window_present (GTK_WINDOW (win5));
 
   win6 = test_chase ();
-  gtk_widget_show (win6);
+  gtk_window_present (GTK_WINDOW (win6));
 
   win7 = test_stacking ();
-  gtk_widget_show (win7);
+  gtk_window_present (GTK_WINDOW (win7));
 
   win8 = test_input_stacking ();
-  gtk_widget_show (win8);
+  gtk_window_present (GTK_WINDOW (win8));
 
   while (TRUE)
     g_main_context_iteration (NULL, TRUE);
